@@ -1,5 +1,5 @@
 /*
- * $Id: gfs.h 9906 2015-09-21 13:28:28Z tatebe $
+ * $Id: gfs.h 10658 2017-11-02 13:02:27Z n-soda $
  */
 
 /*
@@ -122,6 +122,7 @@ gfarm_error_t gfs_pio_open(const char *, int, GFS_File *);
 gfarm_error_t gfs_pio_fhopen(gfarm_ino_t, gfarm_uint64_t, int, GFS_File *);
 gfarm_error_t gfs_pio_create(const char *, int, gfarm_mode_t, GFS_File *);
 gfarm_error_t gfs_pio_close(GFS_File);
+gfarm_error_t gfs_pio_close_getgen(GFS_File gf, gfarm_uint64_t *igenp);
 
 int gfs_pio_eof(GFS_File);
 gfarm_error_t gfs_pio_error(GFS_File);
@@ -136,7 +137,9 @@ gfarm_error_t gfs_pio_datasync(GFS_File);
 gfarm_error_t gfs_pio_truncate(GFS_File, gfarm_off_t);
 gfarm_error_t gfs_pio_pread(GFS_File, void *, int, gfarm_off_t, int *);
 gfarm_error_t gfs_pio_pwrite(GFS_File, void *, int, gfarm_off_t, int *);
-
+typedef gfarm_off_t (*gfs_pageio_t)(char *, gfarm_off_t, int, void*);
+gfarm_error_t gfs_pio_pread_page(GFS_File, gfarm_off_t, int, int,
+		gfs_pageio_t,  void*);
 
 int gfs_pio_getc(GFS_File);
 int gfs_pio_ungetc(GFS_File, int);
@@ -193,6 +196,8 @@ gfarm_error_t gfs_opendirplus(const char *, GFS_DirPlus *);
 gfarm_error_t gfs_closedirplus(GFS_DirPlus);
 gfarm_error_t gfs_readdirplus(GFS_DirPlus,
 	struct gfs_dirent **, struct gfs_stat **);
+gfarm_error_t gfs_seekdirplus(GFS_DirPlus dir, gfarm_off_t off);
+gfarm_error_t gfs_teldirplus(GFS_DirPlus dir, gfarm_off_t *off);
 
 gfarm_error_t gfs_realpath(const char *, char **);
 
@@ -239,6 +244,9 @@ gfarm_error_t gfs_access(const char *, int);
 #define GFS_W_OK	2
 #define GFS_R_OK	4
 
+/* gfarm xattr prefix */
+#define GFARM_EA_PREFIX		"gfarm."
+
 /* 5th argument (flags) of gfs_setxattr() and gfs_fsetxattr() */
 #define GFS_XATTR_CREATE    0x1     /* set value, fail if attr already exists */
 #define GFS_XATTR_REPLACE   0x2     /* set value, fail if attr does not exist */
@@ -317,6 +325,8 @@ gfarm_error_t gfs_replica_list_by_name(const char *, int *, char ***);
 gfarm_error_t gfs_replica_remove_by_file(const char *, const char *);
 gfarm_error_t gfs_replicate_to_local(GFS_File, char *, int);
 
+gfarm_error_t gfs_statfs_by_path(const char *,
+	gfarm_off_t *, gfarm_off_t *, gfarm_off_t *);
 gfarm_error_t gfs_statfs(gfarm_off_t *, gfarm_off_t *, gfarm_off_t *);
 gfarm_error_t gfs_statfsnode_by_path(const char *, char *, int,
 	gfarm_int32_t *, gfarm_off_t *, gfarm_off_t *,
@@ -423,6 +433,9 @@ gfarm_error_t gfs_acl_from_text_with_default(const char *, gfarm_acl_t *,
 gfarm_error_t gfs_acl_delete_mode(gfarm_acl_t);
 #endif
 
+/* Key name of virtual extended attribute for dirquota */
+#define GFARM_EA_DIRECTORY_QUOTA	GFARM_EA_PREFIX "directory_quota"
+
 /*
  * Key names of extended attribute for gfarm_root.*
  */
