@@ -5,6 +5,13 @@
 #include <gfarm/gfarm.h>
 #include "config_openssl.h"
 
+/*
+ * CRYPTO_set_locking_callback() and CRYPTO_set_id_callback() are removed
+ * since OpenSSL-1.1.0.
+ */
+
+#ifdef HAVE_CRYPTO_SET_LOCKING_CALLBACK
+
 static pthread_mutex_t *config_openssl_mutexes;
 
 static void
@@ -16,6 +23,10 @@ gfarm_openssl_lock(int mode, int n, const char *file, int line)
 		pthread_mutex_unlock(&config_openssl_mutexes[n]);
 }
 
+#endif
+
+#ifdef HAVE_CRYPTO_SET_ID_CALLBACK
+
 static unsigned long
 gfarm_openssl_threadid(void)
 {
@@ -23,6 +34,8 @@ gfarm_openssl_threadid(void)
 	return ((unsigned long)pthread_self());
 }
 
+#endif
+
 /*
  * this may be called again after gfarm_terminate(), in which we do
  * not destroy openssl_mutexes since it is considered to be sefe for
@@ -31,13 +44,17 @@ gfarm_openssl_threadid(void)
 void
 gfarm_openssl_initialize()
 {
+#ifdef HAVE_CRYPTO_SET_LOCKING_CALLBACK
 	int num_locks, i;
-	static int initialized = 0;
 	static const char diag[] = "gfarm_openssl_initialize";
+#endif
+	static int initialized = 0;
 
 	if (initialized)
 		return;
 	SSL_library_init();
+
+#ifdef HAVE_CRYPTO_SET_LOCKING_CALLBACK
 	num_locks = CRYPTO_num_locks();
 	GFARM_MALLOC_ARRAY(config_openssl_mutexes, num_locks);
 	if (config_openssl_mutexes == NULL)
@@ -45,6 +62,9 @@ gfarm_openssl_initialize()
 	for (i = 0; i < num_locks; ++i)
 		pthread_mutex_init(&config_openssl_mutexes[i], NULL);
 	CRYPTO_set_locking_callback(gfarm_openssl_lock);
+#endif
+#ifdef HAVE_CRYPTO_SET_ID_CALLBACK
 	CRYPTO_set_id_callback(gfarm_openssl_threadid);
+#endif
 	initialized = 1;
 }
