@@ -8,6 +8,11 @@ struct gfarm_group_info;
 struct gfarm_group_names;
 struct gfarm_quota_get_info;
 struct gfarm_quota_set_info;
+struct gfarm_quota_subject_info;
+struct gfarm_quota_subject_time;
+struct gfarm_quota_limit_info;
+struct gfarm_dirset_info;
+struct gfarm_dirset_dir_info;
 
 struct gfarm_host_sched_info {
 	char *host;
@@ -174,6 +179,9 @@ gfarm_error_t gfm_client_fhopen_request(struct gfm_connection *,
 gfarm_error_t gfm_client_fhopen_result(struct gfm_connection *, gfarm_mode_t *);
 gfarm_error_t gfm_client_close_request(struct gfm_connection *);
 gfarm_error_t gfm_client_close_result(struct gfm_connection *);
+gfarm_error_t gfm_client_close_getgen_request(struct gfm_connection *);
+gfarm_error_t gfm_client_close_getgen_result(struct gfm_connection *,
+		gfarm_uint64_t *);
 gfarm_error_t gfm_client_close_read_request(struct gfm_connection *,
 	gfarm_int64_t, gfarm_int32_t);
 gfarm_error_t gfm_client_close_read_result(struct gfm_connection *);
@@ -326,6 +334,33 @@ gfarm_error_t gfm_client_quota_group_set(struct gfm_connection *,
 					struct gfarm_quota_set_info *);
 gfarm_error_t gfm_client_quota_check(struct gfm_connection *);
 
+gfarm_error_t gfm_client_dirset_info_set(struct gfm_connection *,
+	const char *, const char *);
+gfarm_error_t gfm_client_dirset_info_remove(struct gfm_connection *,
+	const char *, const char *);
+gfarm_error_t gfm_client_dirset_info_list(struct gfm_connection *,
+	const char *, int *, struct gfarm_dirset_info **);
+gfarm_error_t gfm_client_quota_dirset_get(struct gfm_connection *,
+	const char *, const char *,
+	struct gfarm_quota_limit_info *,
+	struct gfarm_quota_subject_info *, struct gfarm_quota_subject_time *,
+	gfarm_uint64_t *);
+gfarm_error_t gfm_client_quota_dirset_set(struct gfm_connection *,
+	const char *, const char *, const struct gfarm_quota_limit_info *);
+gfarm_error_t gfm_client_quota_dir_get_request(struct gfm_connection *);
+gfarm_error_t gfm_client_quota_dir_get_result(struct gfm_connection *,
+	struct gfarm_dirset_info *,
+	struct gfarm_quota_limit_info *,
+	struct gfarm_quota_subject_info *, struct gfarm_quota_subject_time *,
+	gfarm_uint64_t *);
+gfarm_error_t gfm_client_quota_dir_set_request(struct gfm_connection *,
+	const char *, const char *);
+gfarm_error_t gfm_client_quota_dir_set_result(struct gfm_connection *);
+void gfarm_dirset_dir_list_free(int, struct gfarm_dirset_dir_info *);
+gfarm_error_t gfm_client_dirset_dir_list(struct gfm_connection *,
+	const char *, const char *,
+	int *, struct gfarm_dirset_dir_info **);
+
 /* gfs from gfsd */
 gfarm_error_t gfm_client_reopen_request(struct gfm_connection *);
 gfarm_error_t gfm_client_reopen_result(struct gfm_connection *,
@@ -417,6 +452,13 @@ gfarm_error_t gfm_client_replica_check_ctrl_reduced_log_enable(
 	struct gfm_connection *);
 gfarm_error_t gfm_client_replica_check_ctrl_reduced_log_disable(
 	struct gfm_connection *);
+const char *gfm_client_replica_check_status_string(gfarm_int32_t);
+gfarm_error_t gfm_client_replica_check_status_mainctrl(struct gfm_connection *,
+	gfarm_int32_t *);
+gfarm_error_t gfm_client_replica_check_status_remove(struct gfm_connection *,
+	gfarm_int32_t *);
+gfarm_error_t gfm_client_replica_check_status_reduced_log(
+	struct gfm_connection *, gfarm_int32_t *);
 
 /* replica management from gfsd */
 gfarm_error_t gfm_client_replica_adding_request(struct gfm_connection *,
@@ -442,10 +484,11 @@ gfarm_error_t gfm_client_replica_lost_result(struct gfm_connection *);
 gfarm_error_t gfm_client_replica_add_request(struct gfm_connection *,
 	gfarm_ino_t, gfarm_uint64_t, gfarm_off_t);
 gfarm_error_t gfm_client_replica_add_result(struct gfm_connection *);
-gfarm_error_t gfm_client_replica_get_my_entries_request(
-	struct gfm_connection *, gfarm_ino_t, int);
-gfarm_error_t gfm_client_replica_get_my_entries_result(struct gfm_connection *,
-	int *, gfarm_ino_t **, gfarm_uint64_t **, gfarm_off_t **);
+gfarm_error_t gfm_client_replica_get_my_entries_range_request(
+	struct gfm_connection *, gfarm_ino_t, gfarm_ino_t, int);
+gfarm_error_t gfm_client_replica_get_my_entries_range_result(
+	struct gfm_connection *, int *, gfarm_uint32_t *,
+	gfarm_ino_t **, gfarm_uint64_t **, gfarm_off_t **);
 gfarm_error_t gfm_client_replica_create_file_in_lost_found_request(
 	struct gfm_connection *, gfarm_ino_t, gfarm_uint64_t, gfarm_off_t,
 	const struct gfarm_timespec *);
