@@ -63,15 +63,15 @@ enum gfm_proto_command {
 	GFM_PROTO_QUOTA_GROUP_GET,
 	GFM_PROTO_QUOTA_GROUP_SET,
 	GFM_PROTO_QUOTA_CHECK,
-	GFM_PROTO_METADATA_RESERVE5,
-	GFM_PROTO_METADATA_RESERVE6,
-	GFM_PROTO_METADATA_RESERVE7,
-	GFM_PROTO_METADATA_RESERVE8,
-	GFM_PROTO_METADATA_RESERVE9,
-	GFM_PROTO_METADATA_RESERVE10,
-	GFM_PROTO_METADATA_RESERVE11,
-	GFM_PROTO_METADATA_RESERVE12,
-	GFM_PROTO_METADATA_RESERVE13,
+	GFM_PROTO_DIRSET_INFO_GET,		/* not used, but reserved */
+	GFM_PROTO_DIRSET_INFO_SET,		/* since gfarm-2.6.XX */
+	GFM_PROTO_DIRSET_INFO_REMOVE,		/* since gfarm-2.6.XX */
+	GFM_PROTO_DIRSET_INFO_LIST,		/* since gfarm-2.6.XX */
+	GFM_PROTO_QUOTA_DIRSET_GET,		/* since gfarm-2.6.XX */
+	GFM_PROTO_QUOTA_DIRSET_SET,		/* since gfarm-2.6.XX */
+	GFM_PROTO_QUOTA_DIR_GET,		/* since gfarm-2.6.XX */
+	GFM_PROTO_QUOTA_DIR_SET,		/* since gfarm-2.6.XX */
+	GFM_PROTO_DIRSET_DIR_LIST,		/* since gfarm-2.6.XX */
 	GFM_PROTO_METADATA_RESERVE14,
 	GFM_PROTO_METADATA_RESERVE15,
 
@@ -103,7 +103,7 @@ enum gfm_proto_command {
 	GFM_PROTO_VERIFY_TYPE_NOT,
 	GFM_PROTO_REVOKE_GFSD_ACCESS,
 	GFM_PROTO_FHOPEN,
-	GFM_PROTO_FD_MNG_OP_RESERVE9,
+	GFM_PROTO_CLOSE_GETGEN,
 	GFM_PROTO_FD_MNG_OP_RESERVE10,
 	GFM_PROTO_FD_MNG_OP_RESERVE11,
 	GFM_PROTO_FD_MNG_OP_RESERVE12,
@@ -229,7 +229,7 @@ enum gfm_proto_command {
 	GFM_PROTO_REPLICATE_FILE_FROM_TO,
 	GFM_PROTO_REPLICATE_FILE_TO,
 	GFM_PROTO_REPLICA_CHECK_CTRL,
-	GFM_PROTO_REPLICA_OP_RESERVE8,
+	GFM_PROTO_REPLICA_CHECK_STATUS,
 	GFM_PROTO_REPLICA_OP_RESERVE9,
 	GFM_PROTO_REPLICA_OP_RESERVE10,
 	GFM_PROTO_REPLICA_OP_RESERVE11,
@@ -255,7 +255,7 @@ enum gfm_proto_command {
 	GFM_PROTO_REPLICA_OPEN_STATUS,		/* since gfarm-2.6.6 */
 	GFM_PROTO_REPLICA_GET_CKSUM,		/* since gfarm-2.6.8 */
 	GFM_PROTO_FHSET_CKSUM,			/* since gfarm-2.6.8 */
-	GFM_PROTO_REPLICA_MNG_RESERVE15,
+	GFM_PROTO_REPLICA_GET_MY_ENTRIES_RANGE,	/* since gfarm-2.6.21 */
 
 	/* job management */
 
@@ -364,6 +364,19 @@ enum gfm_proto_command {
 #define GFM_PROTO_REPLICA_CHECK_CTRL_REDUCED_LOG_ENABLE		4
 #define GFM_PROTO_REPLICA_CHECK_CTRL_REDUCED_LOG_DISABLE	5
 
+/* GFM_PROTO_REPLICA_CHECK_STATUS request */
+#define GFM_PROTO_REPLICA_CHECK_STATUS_MAINCTRL			0
+#define GFM_PROTO_REPLICA_CHECK_STATUS_REMOVE			1
+#define GFM_PROTO_REPLICA_CHECK_STATUS_REDUCED_LOG		2
+
+/* GFM_PROTO_REPLICA_CHECK_STATUS result */
+#define GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE			0
+#define GFM_PROTO_REPLICA_CHECK_STATUS_DISABLE			1
+#define GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE_RUNNING		2
+#define GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE_STOPPED		3
+#define GFM_PROTO_REPLICA_CHECK_STATUS_DISABLE_RUNNING		4
+#define GFM_PROTO_REPLICA_CHECK_STATUS_DISABLE_STOPPED		5
+
 /*
  * result flags of GFM_PROTO_CKSUM_GET and GFM_PROTO_REPLICA_GET_CKSUM.
  * see cksum request_flags of GFS_PROTO_REPLICATION_CKSUM_REQUEST,
@@ -401,6 +414,11 @@ enum gfm_proto_command {
  * and currently always 0.
  */
 
+/* GFM_PROTO_REPLICA_GET_MY_ENTRIES_RANGE result flags */
+#define GFM_PROTO_REPLICA_GET_MY_ENTRIES_END_OF_INODE	0x00000001
+#define GFM_PROTO_REPLICA_GET_MY_ENTRIES_END_OF_RANGE	0x00000002
+
+
 /*
  * data size limits:
  *
@@ -425,6 +443,8 @@ enum gfm_proto_command {
 
 #define GFARM_GROUP_NAME_MAX			8192 /* VOMS needs long name */
 
+#define GFARM_DIRSET_NAME_MAX			64
+
 /* see GFS_MAXNAMLEN in <gfarm/gfs.h> as well */
 #define GFARM_PATH_MAX				1024
 
@@ -445,6 +465,9 @@ enum gfm_proto_command {
 #define GFARM_XMLATTR_SIZE_MAX_DEFAULT		(768*1024)
 #define GFARM_XMLATTR_SIZE_MAX_LIMIT		(1024*1024-64*1024)
 
+/* flags of GFM_PROTO_QUOTA_DIRSET_GET and GFM_PROTO_QUOTA_DIR_GET */
+#define GFM_PROTO_QUOTA_INACCURATE		1
+
 /* GFM_PROTO_SCHEDULE_FILE, GFM_PROTO_SCHEDULE_FILE_WITH_PROGRAM */
 #define GFM_PROTO_SCHED_FLAG_HOST_AVAIL		1 /* always TRUE for now */
 #define GFM_PROTO_SCHED_FLAG_LOADAVG_AVAIL	2 /* always TRUE for now */
