@@ -1,5 +1,5 @@
 /*
- * $Id: gfs_pio.c 10315 2016-12-03 09:14:51Z tatebe $
+ * $Id: gfs_pio.c 10737 2017-12-06 04:34:43Z tatebe $
  */
 
 #include <sys/types.h> /* mode_t */
@@ -231,6 +231,7 @@ check_connection_in_file_list(GFS_File gf, void *closure)
 #endif
 #endif /* __KERNEL__ */
 
+#ifdef GFARM_USE_OPENSSL
 int
 gfs_pio_md_init(const char *md_type_name, EVP_MD_CTX **md_ctxp, char *url)
 {
@@ -301,6 +302,7 @@ gfs_pio_md_finish(GFS_File gf)
 	}
 	return (GFARM_ERR_NO_ERROR);
 }
+#endif /* GFARM_USE_OPENSSL */
 
 /* only gfs_pio_failover.c:gfs_pio_reopen() is allowed to use this function */
 gfarm_error_t
@@ -392,6 +394,7 @@ gfs_file_alloc(struct gfm_connection *gfm_server, gfarm_int32_t fd, int flags,
 	gf->md.cksum_len = 0;
 	gf->md.cksum_get_flags = 0;
 	gf->md.cksum_set_flags = 0;
+#ifdef GFARM_USE_OPENSSL
 	if (cip == NULL) {
 		/* do not calculate cksum */
 	} else if (!gfs_pio_md_init(cip->cksum_type, &gf->md_ctx, url)) {
@@ -405,6 +408,7 @@ gfs_file_alloc(struct gfm_connection *gfm_server, gfarm_int32_t fd, int flags,
 		    GFM_PROTO_CKSUM_GET_EXPIRED)) == 0)
 			gf->mode |= GFS_FILE_MODE_DIGEST_CALC;
 	}
+#endif /* GFARM_USE_OPENSSL */
 
 	gf->view_context = NULL;
 	gfs_pio_set_view_default(gf);
@@ -442,7 +446,7 @@ gfs_pio_create_igen(const char *url, int flags, gfarm_mode_t mode,
 	struct gfm_connection *gfm_server;
 	int fd, type;
 	gfarm_timerval_t t1, t2;
-	char *real_url;
+	char *real_url = NULL;
 	/* for gfarm_file_trace */
 	int src_port;
 	gfarm_ino_t inum;
@@ -464,8 +468,9 @@ gfs_pio_create_igen(const char *url, int flags, gfarm_mode_t mode,
 			e = gfs_file_alloc(gfm_server, fd, flags, real_url,
 			    inum, gen, cip, gfp);
 		if (e != GFARM_ERR_NO_ERROR) {
+			free(real_url);
 			/* ignore result */
-			(void)gfm_close_fd(gfm_server, fd, NULL);
+			(void)gfm_close_fd(gfm_server, fd, NULL, NULL);
 			gfm_client_connection_free(gfm_server);
 			gflog_debug(GFARM_MSG_1001295,
 				"creation of pio for URL (%s) failed: %s",
@@ -536,7 +541,7 @@ gfs_pio_open(const char *url, int flags, GFS_File *gfp)
 		if (e != GFARM_ERR_NO_ERROR) {
 			free(real_url);
 			/* ignore result */
-			(void)gfm_close_fd(gfm_server, fd, NULL);
+			(void)gfm_close_fd(gfm_server, fd, NULL, NULL);
 			gfm_client_connection_free(gfm_server);
 			gflog_debug(GFARM_MSG_1001297,
 				"open operation on pio for URL (%s) failed: %s",
@@ -568,6 +573,7 @@ gfs_pio_fhopen(gfarm_ino_t inum, gfarm_uint64_t gen, int flags, GFS_File *gfp)
 	    gfarm_ctxp->client_digest_check ? &ci : NULL;
 
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
+	GFARM_KERNEL_UNUSE2(t1, t2);
 	gfs_profile(gfarm_gettimerval(&t1));
 
 	if ((e = gfm_fhopen_fd(inum, gen, flags, &gfm_server, &fd, &type, cip))
@@ -581,7 +587,7 @@ gfs_pio_fhopen(gfarm_ino_t inum, gfarm_uint64_t gen, int flags, GFS_File *gfp)
 			    inum, gen, cip, gfp);
 		if (e != GFARM_ERR_NO_ERROR) {
 			/* ignore result */
-			(void)gfm_close_fd(gfm_server, fd, NULL);
+			(void)gfm_close_fd(gfm_server, fd, NULL, NULL);
 			gfm_client_connection_free(gfm_server);
 		}
 	}
@@ -614,7 +620,7 @@ gfs_pio_get_nfragment(GFS_File gf, int *nfragmentsp)
 static gfarm_error_t gfs_pio_view_fstat(GFS_File, struct gfs_stat *);
 
 gfarm_error_t
-gfs_pio_close(GFS_File gf)
+gfs_pio_close_getgen(GFS_File gf, gfarm_uint64_t *igenp)
 {
 	gfarm_error_t e, e_save;
 	gfarm_timerval_t t1, t2;
@@ -683,6 +689,7 @@ gfs_pio_close(GFS_File gf)
 
 	gfs_pio_file_list_remove(gfarm_filesystem_opened_file_list(fs), gf);
 
+#ifdef GFARM_USE_OPENSSL
 	if ((gf->mode &
 	    (GFS_FILE_MODE_DIGEST_CALC|GFS_FILE_MODE_DIGEST_FINISH)) ==
 	    (GFS_FILE_MODE_DIGEST_CALC) && gf->md_offset == filesize) {
@@ -721,6 +728,7 @@ gfs_pio_close(GFS_File gf)
 		/* not calculated, but need to do this to avoid memory leak */
 		gfarm_msgdigest_free(gf->md_ctx, md_value);
 	}
+#endif /* GFARM_USE_OPENSSL */
 
 	/*
 	 * even if gfsd detectes gfmd failover,
@@ -730,7 +738,7 @@ gfs_pio_close(GFS_File gf)
 	 * closed in gfmd when the connection is closed.
 	 */
 	if (gf->fd >= 0) {
-		e = gfm_close_fd(gf->gfm_server, gf->fd, cip);
+		e = gfm_close_fd(gf->gfm_server, gf->fd, igenp, cip);
 		if (e != GFARM_ERR_NO_ERROR)
 			gflog_debug(GFARM_MSG_1004220,
 				    "gfs_pio_close: close gfmd: %s",
@@ -765,6 +773,11 @@ gfs_pio_close(GFS_File gf)
 
 	return (e_save);
 }
+gfarm_error_t
+gfs_pio_close(GFS_File gf)
+{
+	return (gfs_pio_close_getgen(gf, NULL));
+}
 
 static gfarm_error_t
 gfs_pio_purge(GFS_File gf)
@@ -1088,8 +1101,10 @@ gfs_pio_pread_unbuffer(GFS_File gf, void *buffer, int size,
 		n += length;
 		offset += length;
 		size -= length;
-		if (!length)
+		if (!length) {
+			gf->error = GFARM_ERRMSG_GFS_PIO_IS_EOF;
 			break;
+		}
 	}
 	*np = n;
 	e = GFARM_ERR_NO_ERROR;
@@ -1166,32 +1181,53 @@ gfs_pio_read(GFS_File gf, void *buffer, int size, int *np)
 			gfs_pio_seek(gf, offset + *np, GFARM_SEEK_SET, &result);
 		goto finish;
 	}
-	while (size > 0) {
-		if ((e = gfs_pio_fillbuf(gf, gf->bufsize))
-		    != GFARM_ERR_NO_ERROR) {
-			/* XXX call reconnect, when failover for writing
-			 *     is supported
-			 */
-			if ((gf->mode & GFS_FILE_MODE_READ) == 0 ||
-			    (gf->mode & GFS_FILE_MODE_WRITE) != 0 ||
-			    !IS_CONNECTION_ERROR(e))
+	if (size >= gf->bufsize + (gf->length - gf->p)) {
+		if (gf->p < gf->length) {
+			length = gf->length - gf->p;
+			memcpy(p, gf->buffer + gf->p, length);
+			p += length;
+			n += length;
+			size -= length;
+			gf->p += length;
+		}
+		if ((gf->mode & GFS_FILE_MODE_BUFFER_DIRTY) != 0)
+			e = gfs_pio_flush(gf); /* this does purge too */
+		else
+			e = gfs_pio_purge(gf);
+		if (e == GFARM_ERR_NO_ERROR) {
+			e = gfs_pio_pread_unbuffer(gf, p, size, gf->offset, np);
+			if (e == GFARM_ERR_NO_ERROR) {
+				gf->offset += *np;
+				n += *np;
+			}
+		}
+	} else
+		while (size > 0) {
+			if ((e = gfs_pio_fillbuf(gf, gf->bufsize))
+			    != GFARM_ERR_NO_ERROR) {
+				/* XXX call reconnect, when failover for writing
+				 *     is supported
+				 */
+				if ((gf->mode & GFS_FILE_MODE_READ) == 0 ||
+				    (gf->mode & GFS_FILE_MODE_WRITE) != 0 ||
+				    !IS_CONNECTION_ERROR(e))
+					break;
+				if ((e = gfs_pio_reconnect(gf))
+				    != GFARM_ERR_NO_ERROR)
+					break;
+				continue;
+			}
+			if (gf->error != GFARM_ERR_NO_ERROR) /* EOF or error */
 				break;
-			if ((e = gfs_pio_reconnect(gf))
-			    != GFARM_ERR_NO_ERROR)
-				break;
-			continue;
+			length = gf->length - gf->p;
+			if (length > size)
+				length = size;
+			memcpy(p, gf->buffer + gf->p, length);
+			p += length;
+			n += length;
+			size -= length;
+			gf->p += length;
 		}
-		if (gf->error != GFARM_ERR_NO_ERROR) /* EOF or error */
-			break;
-		length = gf->length - gf->p;
-		if (length > size)
-			length = size;
-		memcpy(p, gf->buffer + gf->p, length);
-		p += length;
-		n += length;
-		size -= length;
-		gf->p += length;
-	}
 	if (e != GFARM_ERR_NO_ERROR) {
 		/*
 		 * when n > 0, part of data is stored in the buffer,
@@ -1210,6 +1246,71 @@ gfs_pio_read(GFS_File gf, void *buffer, int size, int *np)
 
 	return (e);
 }
+gfarm_error_t
+gfs_pio_pread_page(GFS_File gf, gfarm_off_t off, int size, int force,
+		gfs_pageio_t cb, void *arg)
+{
+	gfarm_error_t e;
+	gfarm_timerval_t t1, t2;
+	int second = 0;
+
+	GFARM_KERNEL_UNUSE2(t1, t2);
+	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
+	gfs_profile(gfarm_gettimerval(&t1));
+
+	e = gfs_pio_check_view_default(gf);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1004616,
+			"Check view default for pio failed: %s",
+			gfarm_error_string(e));
+		return (e);
+	}
+
+	CHECK_READABLE(gf);
+
+	do {
+		if (second || off < gf->offset
+			|| off >= (gf->offset + gf->length)) {
+			if (!force)
+				break;
+			gfs_pio_flush(gf);
+			gfs_pio_purge(gf);
+			gf->offset = off;
+
+			if ((e = gfs_pio_fillbuf(gf, gf->bufsize))
+			    != GFARM_ERR_NO_ERROR) {
+				/* XXX call reconnect, when failover for writing
+				 *     is supported
+				 */
+				if ((gf->mode & GFS_FILE_MODE_READ) == 0 ||
+				    (gf->mode & GFS_FILE_MODE_WRITE) != 0 ||
+				    !IS_CONNECTION_ERROR(e))
+					break;
+				if ((e = gfs_pio_reconnect(gf))
+				    != GFARM_ERR_NO_ERROR)
+					break;
+				continue;
+			}
+			if (gf->error != GFARM_ERR_NO_ERROR) /* EOF or error */
+				break;
+		}
+		off = cb(gf->buffer, gf->offset, gf->length, arg);
+		second++;
+	} while (force && off > 0);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1004617,
+			"gfs_pio_fillbuf() failed: %s",
+			gfarm_error_string(e));
+		goto finish;
+	}
+
+	e = GFARM_ERR_NO_ERROR;
+ finish:
+	gfs_profile(gfarm_gettimerval(&t2));
+	gfs_profile(staticp->read_time += gfarm_timerval_sub(&t2, &t1));
+
+	return (e);
+}
 
 gfarm_error_t
 gfs_pio_write(GFS_File gf, const void *buffer, int size, int *np)
@@ -1299,26 +1400,9 @@ gfarm_error_t
 gfs_pio_pread(GFS_File gf, void *buffer, int size, gfarm_off_t offset, int *np)
 {
 	gfarm_error_t e;
-	gfarm_timerval_t t1, t2;
 
-	GFARM_KERNEL_UNUSE2(t1, t2);
-	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
-	gfs_profile(gfarm_gettimerval(&t1));
-
-	e = gfs_pio_check_view_default(gf);
-	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_debug(GFARM_MSG_1003946,
-			"Check view default for pio failed: %s",
-			gfarm_error_string(e));
-		return (e);
-	}
-
-	CHECK_READABLE(gf);
-
-	e = gfs_pio_pread_unbuffer(gf, buffer, size, offset, np);
-
-	gfs_profile(gfarm_gettimerval(&t2));
-	gfs_profile(staticp->read_time += gfarm_timerval_sub(&t2, &t1));
+	if (!(e = gfs_pio_seek(gf, offset, GFARM_SEEK_SET, &offset)))
+		e = gfs_pio_read(gf, buffer, size, np);
 
 	return (e);
 }
@@ -1327,26 +1411,9 @@ gfarm_error_t
 gfs_pio_pwrite(GFS_File gf, void *buffer, int size, gfarm_off_t offset, int *np)
 {
 	gfarm_error_t e;
-	gfarm_timerval_t t1, t2;
 
-	GFARM_KERNEL_UNUSE2(t1, t2);
-	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
-	gfs_profile(gfarm_gettimerval(&t1));
-
-	e = gfs_pio_check_view_default(gf);
-	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_debug(GFARM_MSG_1003947,
-			"gfs_pio_check_view_default() failed: %s",
-			gfarm_error_string(e));
-		return (e);
-	}
-
-	CHECK_WRITABLE(gf);
-
-	e = gfs_pio_pwrite_unbuffer(gf, buffer, size, offset, np);
-
-	gfs_profile(gfarm_gettimerval(&t2));
-	gfs_profile(staticp->write_time += gfarm_timerval_sub(&t2, &t1));
+	if (!(e = gfs_pio_seek(gf, offset, GFARM_SEEK_SET, &offset)))
+		e = gfs_pio_write(gf, buffer, size, np);
 
 	return (e);
 }
