@@ -6,8 +6,8 @@
 #else /* __KERNEL__ */	/* gfs_profile :: never support */
 #define gfs_profile(x)
 #define GFARM_KERNEL_UNUSE(x)	(void)(x)
-#define GFARM_KERNEL_UNUSE2(t1, t2)	(void)(t1), (void)(t2)
-#define GFARM_KERNEL_UNUSE3(t1, t2, t3)	(void)(t1), (void)(t2), (void)(t3)
+#define GFARM_KERNEL_UNUSE2(t1, t2)	((void)(t1), (void)(t2))
+#define GFARM_KERNEL_UNUSE3(t1, t2, t3)	((void)(t1), (void)(t2), (void)(t3))
 #endif /* __KERNEL__ */
 
 void gfs_profile_set(void);
