@@ -1,5 +1,5 @@
 /*
- * $Id: host.c 9952 2015-11-27 07:20:16Z tatebe $
+ * $Id: host.c 10296 2016-11-18 05:43:39Z gfarm-suri $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -41,7 +41,7 @@
 #ifndef __KERNEL__
 #define free_gethost_buff(buf)
 #else /* __KERNEL__ */
-extern void free_gethost_buff(void *buf);
+#define free_gethost_buff(buf) free(buf)
 #endif /* __KERNEL__ */
 
 #define staticp	(gfarm_ctxp->host_static)
