@@ -1,8 +1,9 @@
 /*
- * $Id: liberror.c 9956 2015-11-27 08:08:08Z tatebe $
+ * $Id: liberror.c 10524 2017-06-16 01:37:17Z n-soda $
  */
 
 #include <pthread.h>
+#include <sys/types.h>
 #include <sys/param.h>
 #include <stdlib.h>
 #include <string.h>
@@ -145,6 +146,7 @@ static const char *errcode_string[GFARM_ERR_NUMBER] = {
 	"conflict detected",
 	"invalid credential",
 	"no filesystem node",
+	"directory quota exists",
 };
 
 static const char *errmsg_string[GFARM_ERRMSG_END - GFARM_ERRMSG_BEGIN] = {
@@ -442,6 +444,7 @@ static struct gfarm_errno_error_map {
 	/*		GFARM_ERR_CONFLICT_DETECTED */
 	/*		GFARM_ERR_INVALID_CREDENTIAL */
 	/*		GFARM_ERR_NO_FILESYSTEM_NODE */
+	/*		GFARM_ERR_DIRECTORY_QUOTA_EXISTS */
 };
 
 /* gfarm_error_t -> UNIX errno */
@@ -451,6 +454,7 @@ static struct gfarm_error_errno_map {
 } gfarm_error_errno_map_table[] = {
 	{ EIO,		GFARM_ERR_CHECKSUM_MISMATCH },
 	{ EIO,		GFARM_ERR_NO_FILESYSTEM_NODE },
+	{ ENOTEMPTY,	GFARM_ERR_DIRECTORY_QUOTA_EXISTS },
 };
 
 struct gfarm_error_domain {
@@ -681,13 +685,7 @@ gfarm_error_string(gfarm_error_t error)
 	return (errcode_string[GFARM_ERR_UNKNOWN]);
 }
 
-#ifdef __KERNEL__	/* HAVE_SYS_NERR :: not defined in kernel */
-#undef HAVE_SYS_NERR
-#endif /* __KERNEL__ */
-
-#if defined(HAVE_SYS_NERR)
-# define ERRNO_NUMBER sys_nerr
-#elif defined(ELAST)
+#ifdef ELAST
 # define ERRNO_NUMBER (ELAST + 1)
 #else
 # define ERRNO_NUMBER 256 /* XXX */
@@ -748,9 +746,15 @@ gfarm_errno_to_error(int eno)
 
 	e = gfarm_error_domain_map(gfarm_errno_domain, eno);
 	if (e == GFARM_ERR_UNKNOWN)
+#ifndef __KERNEL__
 		gflog_notice(GFARM_MSG_1003551,
 		    "errno %d:\"%s\" is converted to \"%s\"",
 		    eno, strerror(eno), gfarm_error_string(GFARM_ERR_UNKNOWN));
+#else /* __KERNEL__ */
+		gflog_notice(GFARM_MSG_1003551,
+		    "errno %d: is converted to \"%s\"",
+		    eno, gfarm_error_string(GFARM_ERR_UNKNOWN));
+#endif /* __KERNEL__ */
 	return (e);
 }
 
