@@ -5,36 +5,35 @@
 
 . ./regress.conf
 
-gfs_pio_test=$testbin/../../lib/libgfarm/gfarm/gfs_pio_test/gfs_pio_test
-
-trap 'gfrm -rf $gftmp; rm -f $localtmp; exit $exit_trap' $trap_sigs
-
 $regress/bin/is_digest_enabled || exit $exit_unsupported
 
+gfs_pio_test=$testbin/../../lib/libgfarm/gfarm/gfs_pio_test/gfs_pio_test
+gftmpfile=$gftmp/file
+
+trap 'gfrm -rf $gftmp; rm -f $localtmp; exit $exit_code' 0 $trap_sigs
+exit_code=$exit_trap
+
 if
+   # disable automatic replication
+   gfmkdir $gftmp &&
+   gfncopy -s 1 $gftmp &&
+
    # metadata has checksum
-   gfreg $data/65byte $gftmp &&
-   gfcksum -c $gftmp >/dev/null &&
-   gfrm -f $gftmp &&
+   gfreg $data/65byte $gftmpfile &&
+   gfcksum -c $gftmpfile >/dev/null &&
+   gfrm -f $gftmpfile &&
   
    # metadata does not have checksum
    cat $data/65byte $data/65byte |
-	$gfs_pio_test -ct -O -T 65 $gftmp &&
-   [ X"`gfcksum -t $gftmp`" = X"" ] &&
-   gfcksum -c $gftmp >$localtmp &&
+	$gfs_pio_test -ct -O -T 65 $gftmpfile &&
+   [ X"`gfcksum -t $gftmpfile`" = X"" ] &&
+   gfcksum -c $gftmpfile >$localtmp &&
    type=`awk '{print substr($2, 2, length($2)-2)}' $localtmp` &&
    cksum=`openssl "$type" $data/65byte | awk '{print $NF}'` &&
-   [ X"$cksum" = X"`awk '{print $1}' $localtmp`" ] &&
-   rm -f $localtmp &&
-   gfrm -f $gftmp &&
-
-   true
+   [ X"$cksum" = X"`awk '{print $1}' $localtmp`" ]
 
 then
-  exit $exit_pass
+  exit_code=$exit_pass
 else
-  exit $exit_fail
+  exit_code=$exit_fail
 fi
-
-
-  
