@@ -272,6 +272,7 @@ t_write_cyclic(void)
 		&self_jf, GFARM_JOURNAL_RDWR));
 	reader = journal_file_main_reader(self_jf);
 	writer = journal_file_writer(self_jf);
+	(void)writer; /* shutup gcc warning - set but not used */
 	setup_write();
 	for (i = 0; i < 3; ++i) {
 		ui = t_new_user_info(
@@ -672,6 +673,7 @@ t_write_add_op(void *arg)
 
 	setup_write();
 	writer = journal_file_writer(self_jf);
+	(void)writer; /* shutup gcc warning - set but not used */
 	for (i = 0; i < GFARM_ARRAY_LENGTH(names); ++i) {
 		ui = t_new_user_info(
 		    names[i].username, names[i].realname);
@@ -748,7 +750,7 @@ t_write_blocked(void)
 	 * Note that the writer now starts writing more data.
 	 *
 	 *     |user1|user2|user3|  |
-	 *                 r     
+	 *                 r
 	 */
 	TEST_ASSERT_I("t_write_blocked_num_rec_read",
 	    2, t_write_blocked_num_rec_read);
@@ -3554,6 +3556,9 @@ t_apply(void)
 	char msg[BUFSIZ];
 	int i;
 
+	giant_init();
+	config_var_init();
+
 	unlink_test_file(filepath);
 	TEST_ASSERT_NOERR("journal_file_open",
 	    journal_file_open(filepath, TEST_FILE_MAX_SIZE, 0,
