@@ -3,24 +3,17 @@
 # this test may fail, if number of files which don't have enough
 # replicas are too many.
 
-. ./regress.conf
-tmpf=$gftmp/foo
 base=`dirname $0`
 . ${base}/replica_check-common.sh
-
-check_supported_env
-backup_hostgroup
-trap 'restore_hostgroup; clean_test; exit $exit_trap' $trap_sigs
-clean_test
-setup_test_repattr
+setup_test
 
 set_repattr $NCOPY1 $gftmp
-wait_for_rep $NCOPY1 $tmpf false
+# wait replica_check_minimum_interval
+wait_for_rep $NCOPY1 $tmpf false  "#1 increase"
 
 set_repattr $NCOPY2 $gftmp
-wait_for_rep $NCOPY2 $tmpf false
+# wait replica_check_minimum_interval
+wait_for_rep $NCOPY2 $tmpf false "#2 decrease"
 
-restore_hostgroup
 clean_test
-
 exit $exit_code
