@@ -7,7 +7,9 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <time.h>
 #include <sys/time.h>
+#include <sys/socket.h>
 
 #include <gfarm/gflog.h>
 #include <gfarm/gfarm_config.h>
@@ -16,6 +18,7 @@
 #include <gfarm/gfs.h>
 
 #include "gfutil.h"
+#include "gflog_reduced.h"
 #include "thrsubr.h"
 
 #include "context.h"
@@ -43,7 +46,7 @@ static struct thread_pool *proto_status_send_thread_pool;
 
 static struct peer_watcher *back_channel_recv_watcher;
 
-#define GFS_PROTO_STATUS_TIMEOUT		10000000 /* 10.0 sec. */
+/* SF.net #429: use short timeout, to make GFS_PROTO_STATUS success */
 #define GFS_PROTO_REPLICATION_REQUEST_TIMEOUT	1000000  /*  1.0 sec. */
 #define GFS_PROTO_FHREMOVE_TIMEOUT		100000   /*  0.1 sec. */
 
@@ -229,16 +232,7 @@ gfs_client_status_request(void *arg)
 	host_status_reply_waiting_set(host);
 	e = gfs_client_send_request(host, NULL, diag,
 	    gfs_client_status_result, gfs_client_status_free, host,
-	    GFS_PROTO_STATUS_TIMEOUT, GFS_PROTO_STATUS, "");
-	if (e == GFARM_ERR_DEVICE_BUSY) {
-		host_status_reply_waiting_reset(host);
-		if (!host_status_callout_retry(host)) {
-			gfs_client_status_disconnect_or_message(host, peer,
-			    diag, "request", "status rpc unresponsive");
-		}
-		host_put_peer(host, peer); /* decrement refcount */
-		return (NULL);
-	}
+	    GFM_CLIENT_CHANNEL_TIMEOUT_INFINITY, GFS_PROTO_STATUS, "");
 	if (e != GFARM_ERR_NO_ERROR) {
 		host_status_reply_waiting_reset(host);
 		gflog_error(GFARM_MSG_1001986,
@@ -278,6 +272,17 @@ gfs_client_fhremove_free(void *p, void *arg)
 	removal_finishedq_enqueue(dfc, GFARM_ERR_CONNECTION_ABORTED);
 }
 
+#define SAME_WARNING_TRIGGER	10	/* check reduced mode */
+#define SAME_WARNING_THRESHOLD	30	/* more than this -> reduced mode */
+#define SAME_WARNING_DURATION	600	/* seconds to measure the limit */
+#define	SAME_WARNING_INTERVAL	60	/* seconds: interval of reduced log */
+
+struct gflog_reduced_state busy_state = GFLOG_REDUCED_STATE_INITIALIZER(
+	SAME_WARNING_TRIGGER,
+	SAME_WARNING_THRESHOLD,
+	SAME_WARNING_DURATION,
+	SAME_WARNING_INTERVAL);
+
 static void *
 gfs_client_fhremove_request(void *closure)
 {
@@ -294,7 +299,7 @@ gfs_client_fhremove_request(void *closure)
 	if (e == GFARM_ERR_NO_ERROR) {
 		return (NULL);
 	} else if (e == GFARM_ERR_DEVICE_BUSY) {
-		gflog_info(GFARM_MSG_1002284,
+		gflog_reduced_info(GFARM_MSG_1002284, &busy_state,
 		    "%s(%lld, %lld, %s): "
 		    "busy, waiting for some time", diag,
 		    (long long)dead_file_copy_get_ino(dfc),
@@ -317,7 +322,8 @@ remover(void *closure)
 	for (;;) {
 		dfc = removal_pendingq_dequeue();
 		if (host_is_up(dead_file_copy_get_host(dfc)))
-			thrpool_add_job(back_channel_send_thread_pool,
+			thrpool_add_job_low_priority(
+			    back_channel_send_thread_pool,
 			    gfs_client_fhremove_request, dfc);
 		else /* make this dfcstate_deferred */
 			removal_finishedq_enqueue(dfc,
@@ -776,9 +782,6 @@ gfm_server_switch_back_channel_common(struct peer *peer, int from_client,
 		    "%s: protocol flush: %s",
 		    diag, gfarm_error_string(e2));
 	else if (e == GFARM_ERR_NO_ERROR) {
-		peer_set_async(peer, async);
-		peer_set_watcher(peer, back_channel_recv_watcher);
-
 		if (host_is_up(host)) /* throw away old connetion */ {
 			gflog_warning(GFARM_MSG_1002440,
 			    "back_channel(%s): switching to new connection",
@@ -786,12 +789,22 @@ gfm_server_switch_back_channel_common(struct peer *peer, int from_client,
 			host_disconnect_request(host, NULL);
 		}
 
+		/*
+		 * SF.net #1019
+		 * need giant_lock during transition to back channel
+		 */
 		giant_lock();
+		peer_set_async(peer, async);
+		peer_set_watcher(peer, back_channel_recv_watcher);
 		abstract_host_set_peer(host_to_abstract_host(host),
 		    peer, version);
+
+		(void)gfarm_sockbuf_apply_limit(peer_get_fd(peer),
+		    SO_SNDBUF, gfarm_metadb_server_back_channel_sndbuf_limit,
+		    "metadb_server_back_channel_sndbuf_limit");
+		peer_watch_access(peer);
 		giant_unlock();
 
-		peer_watch_access(peer);
 		callout_setfunc(host_status_callout(host),
 		    proto_status_send_thread_pool,
 		    gfs_client_status_request, host);
@@ -878,11 +891,13 @@ back_channel_init(void)
 		    "%d, queue length:%d: no memory",
 		    gfarm_metadb_thread_pool_size,
 		    gfarm_metadb_job_queue_length);
+	thrpool_set_jobq_low_priority_limit(back_channel_send_thread_pool,
+	    gfarm_metadb_remover_queue_length);
 
 	proto_status_send_thread_pool = thrpool_new(
 	    /* XXX FIXME: use different config parameter */
 	    gfarm_metadb_thread_pool_size, gfarm_metadb_job_queue_length,
-	    "sending to filesystem nodes");
+	    "sending heartbeats to filesystem nodes");
 	if (proto_status_send_thread_pool == NULL)
 		gflog_fatal(GFARM_MSG_1003498,
 		    "GFS_PROTO_STATUS thread pool size:"
