@@ -1,5 +1,5 @@
 /*
- * $Id: fsngroup.h 8365 2013-07-22 02:21:15Z takuya-i $
+ * $Id: fsngroup.h 10665 2017-11-13 10:04:19Z takuya-i $
  */
 
 struct host;
@@ -9,11 +9,13 @@ gfarm_error_t fsngroup_get_hosts(const char *, int *, struct host ***);
  * Replication scheduler:
  */
 struct inode;
+struct dirset;
 struct file_copy;
 gfarm_error_t fsngroup_schedule_replication(
-	struct inode *, const char *, int, struct host **,
+	struct inode *, struct dirset *tdirset,
+	const char *, int, struct host **,
 	int *, struct host **, gfarm_time_t, int *, struct host **,
-	const char *, int *);
+	const char *, int *, int *);
 
 /*
  * Server side RPC stubs:
