@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 10230 2016-09-27 19:13:42Z n-soda $
+ * $Id: gfmd.c 10766 2018-02-06 06:04:05Z n-soda $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -34,6 +34,7 @@
 #include "gflog_reduced.h"
 #include "thrsubr.h"
 
+#include "quota_info.h"
 #include "context.h"
 #include "gfp_xdr.h"
 #include "hostspec.h"
@@ -60,14 +61,16 @@
 #include "inode.h"
 #include "file_copy.h"
 #include "dead_file_copy.h"
+#include "xattr.h"
+#include "quota.h"
+#include "dirset.h"
+#include "quota_dir.h"
+#include "gfmd.h"
 #include "process.h"
 #include "fs.h"
 #include "job.h"
 #include "back_channel.h"
 #include "gfmd_channel.h"
-#include "xattr.h"
-#include "quota.h"
-#include "gfmd.h"
 #include "replica_check.h"
 
 #include "protocol_state.h"
@@ -267,6 +270,9 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_CLOSE:
 		e = gfm_server_close(peer, from_client, skip);
 		break;
+	case GFM_PROTO_CLOSE_GETGEN:
+		e = gfm_server_close_getgen(peer, from_client, skip);
+		break;
 	case GFM_PROTO_VERIFY_TYPE:
 		e = gfm_server_verify_type(peer, from_client, skip);
 		break;
@@ -462,6 +468,9 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_REPLICA_CHECK_CTRL:
 		e = gfm_server_replica_check_ctrl(peer, from_client, skip);
 		break;
+	case GFM_PROTO_REPLICA_CHECK_STATUS:
+		e = gfm_server_replica_check_status(peer, from_client, skip);
+		break;
 	case GFM_PROTO_REPLICA_ADDING:
 		e = gfm_server_replica_adding(peer, from_client, skip,
 		    suspendedp);
@@ -492,6 +501,10 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 		e = gfm_server_replica_get_my_entries2(peer,
 		    from_client, skip);
 		break;
+	case GFM_PROTO_REPLICA_GET_MY_ENTRIES_RANGE:
+		e = gfm_server_replica_get_my_entries_range(peer,
+		    from_client, skip);
+		break;
 	case GFM_PROTO_REPLICA_CREATE_FILE_IN_LOST_FOUND:
 		e = gfm_server_replica_create_file_in_lost_found(peer,
 		    from_client, skip);
@@ -580,6 +593,30 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_QUOTA_CHECK:
 		e = gfm_server_quota_check(peer, from_client, skip);
 		break;
+	case GFM_PROTO_DIRSET_INFO_SET:
+		e = gfm_server_dirset_info_set(peer, from_client, skip);
+		break;
+	case GFM_PROTO_DIRSET_INFO_REMOVE:
+		e = gfm_server_dirset_info_remove(peer, from_client, skip);
+		break;
+	case GFM_PROTO_DIRSET_INFO_LIST:
+		e = gfm_server_dirset_info_list(peer, from_client, skip);
+		break;
+	case GFM_PROTO_QUOTA_DIRSET_GET:
+		e = gfm_server_quota_dirset_get(peer, from_client, skip);
+		break;
+	case GFM_PROTO_QUOTA_DIRSET_SET:
+		e = gfm_server_quota_dirset_set(peer, from_client, skip);
+		break;
+	case GFM_PROTO_QUOTA_DIR_GET:
+		e = gfm_server_quota_dir_get(peer, from_client, skip);
+		break;
+	case GFM_PROTO_QUOTA_DIR_SET:
+		e = gfm_server_quota_dir_set(peer, from_client, skip);
+		break;
+	case GFM_PROTO_DIRSET_DIR_LIST:
+		e = gfm_server_dirset_dir_list(peer, from_client, skip);
+		break;
 	case GFM_PROTO_METADB_SERVER_GET:
 		e = gfm_server_metadb_server_get(peer, from_client, skip);
 		break;
@@ -1213,6 +1250,8 @@ transform_to_master(void)
 	}
 
 	master = mdhost_lookup_master();
+	if (master == NULL)
+		gflog_fatal(GFARM_MSG_1004740, "no master, abort");
 	if (mdhost_is_up(master))
 		mdhost_disconnect_request(master, NULL);
 	gflog_info(GFARM_MSG_1002730,
@@ -1405,6 +1444,7 @@ sigs_handler(void *p)
 		case SIGUSR2:
 			thrpool_info();
 			replica_check_info();
+			replication_info();
 			continue;
 
 		/* some of these will be never delivered due to `*sigs' */
@@ -1467,7 +1507,12 @@ gfmd_terminate(const char *diag)
 	 */
 	peer_shutdown_all();
 
-	gfmdc_journal_transfer_wait();
+	if (gfarm_get_metadb_replication_enabled() &&
+	    gfmd_startup_state_is_ready()) {
+		if (!mdhost_self_is_master())
+			db_journal_cancel_recvq();
+		gfmdc_journal_transfer_wait();
+	}
 
 	/* save all pending transactions */
 	/* db_terminate() needs giant_lock(), see comment in dbq_enter() */
@@ -1504,6 +1549,37 @@ usage(void)
 	exit(1);
 }
 
+static void
+set_db_ops(void)
+{
+	switch (gfarm_backend_db_type) {
+	case GFARM_BACKEND_DB_TYPE_LDAP:
+#ifdef HAVE_LDAP
+		db_use(&db_ldap_ops);
+#else
+		gflog_fatal(GFARM_MSG_1000206,
+		    "LDAP DB is specified, but it's not built in");
+#endif
+		break;
+	case GFARM_BACKEND_DB_TYPE_POSTGRESQL:
+#ifdef HAVE_POSTGRESQL
+		db_use(&db_pgsql_ops);
+#else
+		gflog_fatal(GFARM_MSG_1000207,
+		    "PostgreSQL is specified, but it's not built in");
+#endif
+		break;
+	case GFARM_BACKEND_DB_TYPE_NONE:
+		db_use(&db_none_ops);
+		break;
+	default:
+		gflog_fatal(GFARM_MSG_1000208,
+		    "neither LDAP or PostgreSQL is specified "
+		    "in configuration");
+		break;
+	}
+}
+
 static void
 db_journal_store_failure(void)
 {
@@ -1515,6 +1591,8 @@ db_journal_store_failure(void)
 void
 gfmd_modules_init_default(int table_size)
 {
+	gfarm_error_t e;
+
 	peer_watcher_set_default_nfd(table_size);
 	sync_protocol_watcher = peer_watcher_alloc(
 	    gfarm_metadb_thread_pool_size, gfarm_metadb_job_queue_length,
@@ -1531,11 +1609,22 @@ gfmd_modules_init_default(int table_size)
 	callout_module_init(CALLOUT_NTHREADS);
 
 	if (gfarm_get_metadb_replication_enabled()) {
+		char *diag = "db_journal_init";
+
 		db_journal_set_fail_store_op(db_journal_store_failure);
 		db_journal_apply_init();
-		gflog_info(GFARM_MSG_1004201, "start reading db journal");
-		db_journal_init();
-		boot_apply_db_journal();
+		e = db_journal_init(mdhost_master_disconnect_request);
+		if (gfarm_backend_db_type != GFARM_BACKEND_DB_TYPE_NONE) {
+			if (e != GFARM_ERR_NO_ERROR)
+				gflog_fatal(GFARM_MSG_1004754, "%s: %s",
+					diag, gfarm_error_string(e));
+			boot_apply_db_journal();
+		} else if (e != GFARM_ERR_NO_ERROR) {
+			gflog_error(GFARM_MSG_1004755, "%s: %s, replication "
+				"disabled", diag, gfarm_error_string(e));
+			gfarm_set_metadb_replication_enabled(0);
+			set_db_ops();
+		}
 	}
 	gflog_info(GFARM_MSG_1004202, "start initializing modules and "
 	    "loading database");
@@ -1553,14 +1642,37 @@ gfmd_modules_init_default(int table_size)
 	dir_entry_init();
 	file_copy_init();
 	symlink_init();
+	if (gfarm_backend_db_type == GFARM_BACKEND_DB_TYPE_NONE)
+		xattr_init_cache_all();
 	xattr_init();
 	quota_init();
-
-	/* must be after hosts and filesystem */
-	dead_file_copy_init(mdhost_self_is_master());
+	dirset_init();
+	quota_dir_init();
 
 	peer_init(table_size);
 	job_table_init(table_size);
+
+	if (gfarm_get_metadb_replication_enabled() &&
+	    gfarm_backend_db_type == GFARM_BACKEND_DB_TYPE_NONE) {
+		/* update in-memory metadata by journal */
+		if ((e = create_detached_thread(db_journal_apply_thread, NULL))
+		    != GFARM_ERR_NO_ERROR)
+			gflog_fatal(GFARM_MSG_1004741,
+			    "create_detached_thread(db_journal_apply_thread): "
+			    "%s", gfarm_error_string(e));
+		db_journal_wait_for_apply_thread();
+		/* this updates seqnum */
+		boot_apply_db_journal();
+	}
+	/* initial entry */
+	mdhost_initial_entry();
+	user_initial_entry();
+	group_initial_entry();
+	inode_initial_entry();
+	dir_entry_initial_entry();
+
+	/* must be after hosts, filesystem and mdhost_initial_entry */
+	dead_file_copy_init(mdhost_self_is_master());
 }
 
 /* this interface is made as a hook for a private extension */
@@ -1666,34 +1778,13 @@ main(int argc, char **argv)
 	write_pid();
 
 	giant_init();
+	config_var_init();
 
 	/*
 	 * We do this before calling gfarm_daemon()
 	 * to print the error message to stderr.
 	 */
-	switch (gfarm_backend_db_type) {
-	case GFARM_BACKEND_DB_TYPE_LDAP:
-#ifdef HAVE_LDAP
-		db_use(&db_ldap_ops);
-#else
-		gflog_fatal(GFARM_MSG_1000206,
-		    "LDAP DB is specified, but it's not built in");
-#endif
-		break;
-	case GFARM_BACKEND_DB_TYPE_POSTGRESQL:
-#ifdef HAVE_POSTGRESQL
-		db_use(&db_pgsql_ops);
-#else
-		gflog_fatal(GFARM_MSG_1000207,
-		    "PostgreSQL is specified, but it's not built in");
-#endif
-		break;
-	default:
-		gflog_fatal(GFARM_MSG_1000208,
-		    "neither LDAP or PostgreSQL is specified "
-		    "in configuration");
-		break;
-	}
+	set_db_ops();
 
 	/*
 	 * This initializes dbq, thus, should be called before
@@ -1712,6 +1803,8 @@ main(int argc, char **argv)
 		if (gfarm_daemon(0, 0) == -1)
 			gflog_warning_errno(GFARM_MSG_1001487, "daemon");
 	}
+	giant_type_log();
+
 	/*
 	 * We do this after calling gfarm_daemon(),
 	 * because it changes pid.
