@@ -5,12 +5,15 @@
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
+#include <time.h>
+#include <sys/types.h>
 
 #include <gfarm/gfarm.h>
 
 #include "gfutil.h"
 #include "hash.h"
 
+#include "quota_info.h"
 #include "context.h"
 #include "gfp_xdr.h"
 #include "auth.h"
@@ -30,7 +33,7 @@ struct group {
 	char *groupname;
 	struct group_assignment users;
 	struct quota quota;
-	struct usage usage_tmp;
+	struct gfarm_quota_subject_info usage_tmp;
 	int invalid;	/* set when deleted */
 };
 
@@ -278,15 +281,15 @@ group_quota(struct group *g)
 	return (&g->quota);
 }
 
-struct usage *
+struct gfarm_quota_subject_info *
 group_usage_tmp(struct group *g)
 {
 	return (&g->usage_tmp);
 }
 
 void
-group_all(void *closure, void (*callback)(void *, struct group *),
-	  int valid_only)
+group_foreach(void *closure, void (*callback)(void *, struct group *),
+	int valid_only)
 {
 	struct gfarm_hash_iterator it;
 	struct group **g;
@@ -428,9 +431,6 @@ void
 group_init(void)
 {
 	gfarm_error_t e;
-	struct group *admin;
-	struct gfarm_group_info gi;
-	static const char diag[] = "group_init";
 
 	group_hashtab =
 	    gfarm_hash_table_alloc(GROUP_HASHTAB_SIZE,
@@ -442,6 +442,15 @@ group_init(void)
 	if (e != GFARM_ERR_NO_ERROR)
 		gflog_error(GFARM_MSG_1000248,
 		    "loading groups: %s", gfarm_error_string(e));
+}
+
+void
+group_initial_entry(void)
+{
+	gfarm_error_t e;
+	struct group *admin;
+	struct gfarm_group_info gi;
+	static const char diag[] = "group_init";
 
 	if ((admin = group_lookup(ADMIN_GROUP_NAME)) == NULL) {
 		gflog_info(GFARM_MSG_1000249,
@@ -815,9 +824,6 @@ gfm_server_group_info_set(struct peer *peer, int from_client, int skip)
 		gfarm_group_info_free(&gi);
 	giant_unlock();
 
-	if (e == GFARM_ERR_NO_ERROR)
-		quota_check_start();
-
 	return (gfm_server_put_reply(peer, diag, e, ""));
 }
 
