@@ -1,5 +1,5 @@
 /*
- * $Id: host.c 10207 2016-08-24 01:56:26Z tatebe $
+ * $Id: host.c 10718 2017-12-02 17:56:06Z n-soda $
  */
 
 #include <assert.h>
@@ -31,6 +31,7 @@
 #include "gfs_proto.h" /* GFS_PROTOCOL_VERSION */
 #include "auth.h"
 #include "config.h"
+#include "context.h"
 
 #include "callout.h"
 #include "subr.h"
@@ -70,7 +71,7 @@ struct host {
 	 */
 	char *fsngroupname;
 
-	pthread_mutex_t back_channel_mutex;
+	/* the following members are protected by abstract_host_mutex */
 
 #ifdef COMPAT_GFARM_2_3
 	/* used by synchronous protocol (i.e. until gfarm-2.3.0) only */
@@ -112,7 +113,7 @@ static struct host *host_new(struct gfarm_host_info *, struct callout *);
 static void host_free(struct host *);
 
 /* NOTE: each entry should be checked by host_is_valid(h) too */
-#define FOR_ALL_HOSTS(it) \
+#define HOST_FOREACH(it) \
 	for (gfarm_hash_iterator_begin(host_hashtab, (it)); \
 	    !gfarm_hash_iterator_is_end(it); \
 	     gfarm_hash_iterator_next(it))
@@ -266,7 +267,7 @@ host_addr_lookup(const char *hostname, struct sockaddr *addr)
 
 	/* XXX FIXME - this is too damn slow */
 
-	FOR_ALL_HOSTS(&it) {
+	HOST_FOREACH(&it) {
 		h = host_iterator_access(&it);
 		if (!host_is_valid(h))
 			continue;
@@ -518,19 +519,8 @@ host_supports_cksum_protocols(struct host *h)
 		>= GFS_PROTOCOL_VERSION_V2_6);
 }
 
-static void
-back_channel_mutex_lock(struct host *h, const char *diag)
-{
-	gfarm_mutex_lock(&h->back_channel_mutex, diag, BACK_CHANNEL_DIAG);
-}
-
-static void
-back_channel_mutex_unlock(struct host *h, const char *diag)
-{
-	gfarm_mutex_unlock(&h->back_channel_mutex, diag, BACK_CHANNEL_DIAG);
-}
-
 #ifdef COMPAT_GFARM_2_3
+
 void
 host_set_callback(struct abstract_host *ah, struct peer *peer,
 	result_callback_t result_callback,
@@ -541,12 +531,12 @@ host_set_callback(struct abstract_host *ah, struct peer *peer,
 	static const char diag[] = "host_set_callback";
 
 	/* XXX FIXME sanity check? */
-	back_channel_mutex_lock(h, diag);
+	abstract_host_mutex_lock(ah, diag);
 	h->back_channel_result = result_callback;
 	h->back_channel_disconnect = disconnect_callback;
 	h->back_channel_callback_peer = peer;
 	h->back_channel_callback_closure = closure;
-	back_channel_mutex_unlock(h, diag);
+	abstract_host_mutex_unlock(ah, diag);
 }
 
 int
@@ -556,7 +546,7 @@ host_get_result_callback(struct host *h, struct peer *peer,
 	int ok;
 	static const char diag[] = "host_get_result_callback";
 
-	back_channel_mutex_lock(h, diag);
+	abstract_host_mutex_lock(&h->ah, diag);
 
 	if (h->back_channel_result == NULL ||
 	    h->back_channel_callback_peer != peer) {
@@ -571,7 +561,7 @@ host_get_result_callback(struct host *h, struct peer *peer,
 		ok = 1;
 	}
 
-	back_channel_mutex_unlock(h, diag);
+	abstract_host_mutex_unlock(&h->ah, diag);
 	return (ok);
 }
 
@@ -583,7 +573,7 @@ host_get_disconnect_callback(struct host *h,
 	int ok;
 	static const char diag[] = "host_get_disconnect_callback";
 
-	back_channel_mutex_lock(h, diag);
+	abstract_host_mutex_lock(&h->ah, diag);
 
 	if (h->back_channel_disconnect == NULL) {
 		ok = 0;
@@ -598,14 +588,14 @@ host_get_disconnect_callback(struct host *h,
 		ok = 1;
 	}
 
-	back_channel_mutex_unlock(h, diag);
+	abstract_host_mutex_unlock(&h->ah, diag);
 	return (ok);
 }
 
-#endif
+#endif /* COMPAT_GFARM_2_3 */
 
 /*
- * PREREQUISITE: host::back_channel_mutex
+ * PREREQUISITE: abstract_host::mutex
  * LOCKS: nothing
  * SLEEPS: no
  */
@@ -617,7 +607,7 @@ host_is_up_unlocked(struct host *h)
 
 /*
  * PREREQUISITE: nothing
- * LOCKS: host::back_channel_mutex
+ * LOCKS: host::acstract_host::mutex
  * SLEEPS: no
  */
 int
@@ -637,25 +627,50 @@ host_is_up_with_grace(struct host *h, gfarm_time_t grace)
 	if (grace <= 0)
 		return (0);
 
-	back_channel_mutex_lock(h, diag);
+	abstract_host_mutex_lock(&h->ah, diag);
 	rv = h->disconnect_time + grace > time(NULL) ? 1 : 0;
-	back_channel_mutex_unlock(h, diag);
+	abstract_host_mutex_unlock(&h->ah, diag);
 	return (rv);
 }
 
+static int
+host_is_busy_unlocked(struct host *h)
+{
+	long long load = 0, busy = 0;
+
+	if (host_is_up_unlocked(h)) {
+		load = h->status.loadavg_1min * GFARM_F2LL_SCALE;
+		busy = h->hi.ncpu * gfarm_ctxp->schedule_busy_load;
+	}
+	return (load >= busy);
+}
+
+static int
+host_is_busy(struct host *h)
+{
+	int busy = 0;
+	static const char diag[] = "host_is_busy";
+
+	abstract_host_mutex_lock(&h->ah, diag);
+	busy = host_is_busy_unlocked(h);
+	abstract_host_mutex_unlock(&h->ah, diag);
+
+	return (busy);
+}
+
 int
 host_is_disk_available(struct host *h, gfarm_off_t size)
 {
 	gfarm_off_t avail, minfree = gfarm_get_minimum_free_disk_space();
 	static const char diag[] = "host_is_disk_available";
 
-	back_channel_mutex_lock(h, diag);
+	abstract_host_mutex_lock(&h->ah, diag);
 
 	if (host_is_up_unlocked(h))
-		avail = h->status.disk_avail * 1024;
+		avail = h->status.disk_avail_in_byte;
 	else
 		avail = 0;
-	back_channel_mutex_unlock(h, diag);
+	abstract_host_mutex_unlock(&h->ah, diag);
 
 	/* to reduce no space risk, keep minimum disk space */
 	return (avail >= minfree + size);
@@ -715,9 +730,9 @@ host_status_reply_waiting_set(struct host *host)
 {
 	static const char diag[] = "host_status_reply_waiting_set";
 
-	back_channel_mutex_lock(host, diag);
+	abstract_host_mutex_lock(&host->ah, diag);
 	host->status_reply_waiting = 1;
-	back_channel_mutex_unlock(host, diag);
+	abstract_host_mutex_unlock(&host->ah, diag);
 }
 
 void
@@ -725,9 +740,9 @@ host_status_reply_waiting_reset(struct host *host)
 {
 	static const char diag[] = "host_status_reply_waiting_reset";
 
-	back_channel_mutex_lock(host, diag);
+	abstract_host_mutex_lock(&host->ah, diag);
 	host->status_reply_waiting = 0;
-	back_channel_mutex_unlock(host, diag);
+	abstract_host_mutex_unlock(&host->ah, diag);
 }
 
 int
@@ -736,9 +751,9 @@ host_status_reply_is_waiting(struct host *host)
 	int waiting;
 	static const char diag[] = "host_status_reply_is_waiting";
 
-	back_channel_mutex_lock(host, diag);
+	abstract_host_mutex_lock(&host->ah, diag);
 	waiting = host->status_reply_waiting;
-	back_channel_mutex_unlock(host, diag);
+	abstract_host_mutex_unlock(&host->ah, diag);
 
 	return (waiting);
 }
@@ -765,15 +780,17 @@ void
 host_status_update(struct host *host, struct host_status *status)
 {
 	gfarm_uint64_t saved_used = 0, saved_avail = 0;
+	int saved_busy = 0, busy = 0;
 	const char diag[] = "status_update";
 
-	back_channel_mutex_lock(host, diag);
+	abstract_host_mutex_lock(&host->ah, diag);
 
 	host->status_reply_waiting = 0;
 
 	if (host->report_flags & GFM_PROTO_SCHED_FLAG_LOADAVG_AVAIL) {
 		saved_used = host->status.disk_used;
 		saved_avail = host->status.disk_avail;
+		saved_busy = host_is_busy_unlocked(host);
 	}
 
 	host->last_report = time(NULL);
@@ -783,11 +800,18 @@ host_status_update(struct host *host, struct host_status *status)
 	host->status = *status;
 	host->status.disk_used_in_byte = status->disk_used * 1024;
 	host->status.disk_avail_in_byte = status->disk_avail * 1024;
+	busy = host_is_busy_unlocked(host);
 
-	back_channel_mutex_unlock(host, diag);
+	abstract_host_mutex_unlock(&host->ah, diag);
 
 	host_total_disk_update(saved_used, saved_avail,
 	    status->disk_used, status->disk_avail);
+
+	if (saved_busy && !busy)
+		replica_check_start_host_is_not_busy();
+	if (saved_busy != busy)
+		gflog_info(GFARM_MSG_1004747, "%s becomes %s", host_name(host),
+		    busy ? "busy" : "not busy");
 }
 
 void
@@ -796,12 +820,24 @@ host_status_get_disk_usage(struct host *host,
 {
 	const char diag[] = "host_status_get_disk_usage";
 
-	back_channel_mutex_lock(host, diag);
+	abstract_host_mutex_lock(&host->ah, diag);
 	if (used)
 		*used = host->status.disk_used_in_byte;
 	if (avail)
 		*avail = host->status.disk_avail_in_byte;
-	back_channel_mutex_unlock(host, diag);
+	abstract_host_mutex_unlock(&host->ah, diag);
+}
+
+/* 0 - 100 % */
+float
+host_status_get_disk_usage_percent(struct host *host)
+{
+	gfarm_off_t used;
+	gfarm_off_t avail;
+
+	host_status_get_disk_usage(host, &used, &avail);
+
+	return (100 * (float)used / (float)(used + avail));
 }
 
 void
@@ -809,17 +845,19 @@ host_status_update_disk_usage(struct host *host, gfarm_off_t filesize)
 {
 	const char diag[] = "host_status_update_disk_usage";
 
-	back_channel_mutex_lock(host, diag);
-	host->status.disk_used_in_byte += filesize ;
+	abstract_host_mutex_lock(&host->ah, diag);
+	host->status.disk_used_in_byte += filesize;
 	host->status.disk_avail_in_byte -= filesize;
-	back_channel_mutex_unlock(host, diag);
+	host->status.disk_used = host->status.disk_used_in_byte / 1024;
+	host->status.disk_avail = host->status.disk_avail_in_byte / 1024;
+	abstract_host_mutex_unlock(&host->ah, diag);
 }
 
 /*
  * PREREQUISITE: giant_lock
- * LOCKS: host::back_channel_mutex, dfc_allq.mutex, removal_pendingq.mutex
+ * LOCKS: abstract_host::mutex, dfc_allq.mutex, removal_pendingq.mutex
  * SLEEPS: maybe (see the comment of dead_file_copy_host_becomes_up())
- *	but host::back_channel_mutex, dfc_allq.mutex and removal_pendingq.mutex
+ *	but abstract_host::mutex, dfc_allq.mutex and removal_pendingq.mutex
  *	won't be blocked while sleeping.
  */
 static void
@@ -844,7 +882,7 @@ host_set_peer_unlocked(struct abstract_host *ah, struct peer *p)
 }
 
 /*
- * PREREQUISITE: host::back_channel_mutex
+ * PREREQUISITE: abstract_host::mutex
  * LOCKS: removal_pendingq.mutex, host_busyq.mutex
  * SLEEPS: no
  */
@@ -854,14 +892,14 @@ host_unset_peer(struct abstract_host *ah, struct peer *peer)
 	callout_stop(abstract_host_to_host(ah)->status_callout);
 }
 
+/*
+ * PREREQUISITE: abstract_host::mutex
+ */
 static void
 host_disable(struct abstract_host *ah)
 {
 	struct host *h = abstract_host_to_host(ah);
 	gfarm_uint64_t saved_used, saved_avail;
-	static const char diag[] = "host_disable";
-
-	back_channel_mutex_lock(h, diag);
 
 	if (h->report_flags & GFM_PROTO_SCHED_FLAG_LOADAVG_AVAIL) {
 		saved_used = h->status.disk_used;
@@ -873,19 +911,42 @@ host_disable(struct abstract_host *ah)
 	h->report_flags = 0;
 	h->disconnect_time = time(NULL);
 
-	back_channel_mutex_unlock(h, diag);
-
 	host_total_disk_update(saved_used, saved_avail, 0, 0);
-	replica_check_start_host_down();
 }
 
 static void
 host_disabled(struct abstract_host *ah, struct peer *peer)
 {
+	replica_check_start_host_down();
 	dead_file_copy_host_becomes_down(abstract_host_to_host(ah));
 }
 
-/* giant_lock should be held before calling this */
+/*
+ * PREREQUISITE: nothing
+ * LOCKS:
+ *  - abstract_host::mutex
+ *    in abstract_host_disconnect_request()
+ *    in abstract_host::ops::disable() == host_disable()
+ *    which is called from abstract_host::ops::disable() == host_disable()
+ *  - callout_module::mutex
+ *    in host_unset_peer()
+ *    which is called from abstract_host_peer_unset()
+ *    which is called from abstract_host_disconnect_request()
+ *  - total_disk_mutex
+ *    in abstract_host::ops::disable() == host_disable()
+ *    which is called from abstract_host::ops::disable() == host_disable()
+ *  - replica_check_mutex and config_var_mutex
+ *    in replica_check_signal_host_down()
+ *    which is called from abstract_host::ops::disable() == host_disable()
+ *  - removal_pendingq:mutex -> dead_file_copy::mutex
+ *    in abstract_host::ops::disabled() == host_disabled()
+ *    which is called from abstract_host_disconnect_request()
+ *  - peer_closing_queue.mutex
+ *    in peer_del_ref()
+ *    which is called from abstract_host_disconnect_request()
+ *
+ * NOTE: peer may be NULL.
+ */
 void
 host_disconnect_request(struct host *h, struct peer *peer)
 {
@@ -915,10 +976,9 @@ host_new(struct gfarm_host_info *hi, struct callout *callout)
 	GFARM_MALLOC(h);
 	if (h == NULL)
 		return (NULL);
-	abstract_host_init(&h->ah, &host_ops, "host_new");
+	abstract_host_init(&h->ah, &host_ops, diag);
 	h->hi = *hi;
 	h->fsngroupname = NULL;
-	gfarm_mutex_init(&h->back_channel_mutex, diag, BACK_CHANNEL_DIAG);
 #ifdef COMPAT_GFARM_2_3
 	h->back_channel_result = NULL;
 	h->back_channel_disconnect = NULL;
@@ -1153,6 +1213,19 @@ host_is_disk_available_filter(struct host *host, void *closure)
 	return (host_is_disk_available(host, *sizep));
 }
 
+/*
+ * PREREQUISITE: giant_lock (for gfarm_replication_busy_host)
+ * LOCKS:
+ *  - abstract_host::mutex
+ *    in host_is_busy() and host_is_disk_available()
+ */
+gfarm_error_t
+host_is_not_busy_and_disk_available_filter(struct host *host, void *closure)
+{
+	return ((gfarm_replication_busy_host || !host_is_busy(host))
+		&& host_is_disk_available_filter(host, closure));
+}
+
 gfarm_error_t
 host_array_alloc(int *nhostsp, struct host ***hostsp)
 {
@@ -1161,7 +1234,7 @@ host_array_alloc(int *nhostsp, struct host ***hostsp)
 	struct gfarm_hash_iterator it;
 
 	nhosts = 0;
-	FOR_ALL_HOSTS(&it) {
+	HOST_FOREACH(&it) {
 		h = host_iterator_access(&it);
 		if (host_is_valid(h))
 			++nhosts;
@@ -1172,7 +1245,7 @@ host_array_alloc(int *nhostsp, struct host ***hostsp)
 		return (GFARM_ERR_NO_MEMORY);
 
 	i = 0;
-	FOR_ALL_HOSTS(&it) {
+	HOST_FOREACH(&it) {
 		if (i >= nhosts) /* always false due to giant_lock */
 			break;
 		h = host_iterator_access(&it);
@@ -1208,7 +1281,7 @@ host_from_all(int (*filter)(struct host *, void *), void *closure,
 
 /*
  * PREREQUISITE: giant_lock
- * LOCKS: host::back_channel_mutex
+ * LOCKS: abstract_host::mutex
  * SLEEPS: no
  */
 int
@@ -1218,7 +1291,7 @@ host_number()
 	struct host *h;
 	int nhosts = 0;
 
-	FOR_ALL_HOSTS(&it) {
+	HOST_FOREACH(&it) {
 		h = host_iterator_access(&it);
 		if (host_is_valid(h))
 			++nhosts;
@@ -1243,6 +1316,23 @@ select_hosts(int nhosts, struct host **hosts,
 	}
 }
 
+/*
+ * just select randomly		XXX FIXME: needs to improve
+ */
+int
+host_select_one(int nhosts, struct host **hosts, const char *diag)
+{
+	if (nhosts > 1) {
+		return (gfarm_random() % nhosts);
+	}
+	if (nhosts <= 0) {
+		/* shouldn't happen */
+		gflog_warning(GFARM_MSG_1005012, "%s: nhosts=%d",
+		    diag, nhosts);
+	}
+	return (0);
+}
+
 /*
  * this function modifies *n_exceptionsp and exceptions[],
  * but they may be abled to be used later.
@@ -1457,9 +1547,9 @@ host_init(void)
 
 /*
  * PREREQUISITE: giant_lock
- * LOCKS: host::back_channel_mutex
+ * LOCKS: abstract_host::mutex
  * SLEEPS: maybe
- *	but host::back_channel_mutex won't be blocked while sleeping.
+ *	but abstract_host::mutex won't be blocked while sleeping.
  */
 static gfarm_error_t
 gfm_server_host_generic_get(struct peer *peer,
@@ -1474,7 +1564,7 @@ gfm_server_host_generic_get(struct peer *peer,
 	char *match;
 
 	nhosts = 0;
-	FOR_ALL_HOSTS(&it) {
+	HOST_FOREACH(&it) {
 		h = host_iterator_access(&it);
 		++nhosts;
 	}
@@ -1491,7 +1581,7 @@ gfm_server_host_generic_get(struct peer *peer,
 		    "%s: no memory for %d hosts", diag, nhosts);
 	} else {
 		i = 0;
-		FOR_ALL_HOSTS(&it) {
+		HOST_FOREACH(&it) {
 			if (i >= nhosts) /* always false due to giant_lock */
 				break;
 			h = host_iterator_access(&it);
@@ -1516,7 +1606,7 @@ gfm_server_host_generic_get(struct peer *peer,
 	/* if network error doesn't happen, e2 == e here */
 	if (e2 == GFARM_ERR_NO_ERROR) {
 		i = answered = 0;
-		FOR_ALL_HOSTS(&it) {
+		HOST_FOREACH(&it) {
 			if (i >= nhosts || answered >= nmatch)
 				break;
 			h = host_iterator_access(&it);
@@ -1995,11 +2085,11 @@ host_schedule_reply(struct host *h, struct peer *peer, const char *diag)
 	gfarm_time_t last_report;
 	gfarm_int32_t report_flags;
 
-	back_channel_mutex_lock(h, diag);
+	abstract_host_mutex_lock(&h->ah, diag);
 	status = h->status;
 	last_report = h->last_report;
 	report_flags = h->report_flags;
-	back_channel_mutex_unlock(h, diag);
+	abstract_host_mutex_unlock(&h->ah, diag);
 	return (gfp_xdr_send(peer_get_conn(peer), "siiillllii",
 	    h->hi.hostname, h->hi.port, h->hi.ncpu,
 	    (gfarm_int32_t)(status.loadavg_1min * GFM_PROTO_LOADAVG_FSCALE),
@@ -2141,7 +2231,7 @@ host_iterate(
 	size_t sz;
 
 	nhosts = 0;
-	FOR_ALL_HOSTS(&it) {
+	HOST_FOREACH(&it) {
 		h = host_iterator_access(&it);
 		if (host_is_valid(h))
 			nhosts++;
@@ -2159,7 +2249,7 @@ host_iterate(
 
 	nmatches = 0;
 	p = ret;
-	FOR_ALL_HOSTS(&it) {
+	HOST_FOREACH(&it) {
 		if (nmatches >= nhosts)
 			break;
 		h = host_iterator_access(&it);
