@@ -1,5 +1,5 @@
 /*
- * $Id: host.h 9762 2015-06-24 09:59:35Z takuya-i $
+ * $Id: host.h 10597 2017-09-29 03:18:54Z takuya-i $
  */
 
 void host_init(void);
@@ -67,12 +67,14 @@ void host_intersect(int *, struct host **, int *, struct host **);
 gfarm_error_t host_except(int *, struct host **, int *, struct host **,
 	int (*)(struct host *, void *), void *);
 
+gfarm_error_t host_is_not_busy_and_disk_available_filter(struct host *, void *);
 gfarm_error_t host_is_disk_available_filter(struct host *, void *);
 gfarm_error_t host_array_alloc(int *, struct host ***);
 gfarm_error_t host_from_all(int (*)(struct host *, void *), void *,
 	gfarm_int32_t *, struct host ***);
 int host_number();
 
+int host_select_one(int, struct host **, const char *);
 gfarm_error_t host_from_all_except(int *, struct host **,
 	int (*)(struct host *, void *),	void *,
 	gfarm_int32_t *, struct host ***);
@@ -88,6 +90,7 @@ int host_status_reply_is_waiting(struct host *);
 void host_status_update(struct host *, struct host_status *);
 
 void host_status_get_disk_usage(struct host *, gfarm_off_t *, gfarm_off_t *);
+float host_status_get_disk_usage_percent(struct host *);
 void host_status_update_disk_usage(struct host *, gfarm_off_t);
 
 struct gfarm_host_info;
