@@ -59,9 +59,9 @@ struct mdhost {
 };
 static const char MDHOST_MUTEX_DIAG[]		= "mdhost_mutex";
 
-/* global mutex is for mdhost_hashtab */
-pthread_mutex_t mdhost_global_mutex = PTHREAD_MUTEX_INITIALIZER;
-static const char MDHOST_GLOBAL_MUTEX_DIAG[]	= "mdhost_global_mutex";
+/* mdhost_table_rwlock is for mdhost_hashtab */
+pthread_rwlock_t mdhost_table_rwlock = PTHREAD_RWLOCK_INITIALIZER;
+static const char MDHOST_TABLE_RWLOCK_DIAG[]	= "mdhost_table_rwlock";
 static struct gfarm_hash_table *mdhost_hashtab;
 
 /* no need to protect since it is only updated in mdhost_init() */
@@ -124,16 +124,24 @@ mdhost_mutex_unlock(struct mdhost *m, const char *diag)
 }
 
 static void
-mdhost_global_mutex_lock(const char *diag)
+mdhost_table_rwlock_rdlock(const char *diag)
 {
-	gfarm_mutex_lock(&mdhost_global_mutex, diag, MDHOST_GLOBAL_MUTEX_DIAG);
+	gfarm_rwlock_rdlock(&mdhost_table_rwlock,
+	    diag, MDHOST_TABLE_RWLOCK_DIAG);
 }
 
 static void
-mdhost_global_mutex_unlock(const char *diag)
+mdhost_table_rwlock_wrlock(const char *diag)
 {
-	gfarm_mutex_unlock(&mdhost_global_mutex, diag,
-		MDHOST_GLOBAL_MUTEX_DIAG);
+	gfarm_rwlock_wrlock(&mdhost_table_rwlock,
+	    diag, MDHOST_TABLE_RWLOCK_DIAG);
+}
+
+static void
+mdhost_table_rwlock_unlock(const char *diag)
+{
+	gfarm_rwlock_unlock(&mdhost_table_rwlock,
+	    diag, MDHOST_TABLE_RWLOCK_DIAG);
 }
 
 static void
@@ -275,13 +283,13 @@ mdhost_foreach(int (*func)(struct mdhost *, void *), void *closure)
 	struct mdhost *self = mdhost_lookup_self();
 	static const char diag[] = "mdhost_foreach";
 
-	mdhost_global_mutex_lock(diag);
+	mdhost_table_rwlock_rdlock(diag);
 	FOREACH_MDHOST(it) {
 		m = mdhost_iterator_access(&it);
 		if (mdhost_is_valid(m) && m != self && func(m, closure) == 0)
 			break;
 	}
-	mdhost_global_mutex_unlock(diag);
+	mdhost_table_rwlock_unlock(diag);
 }
 
 static int
@@ -540,10 +548,10 @@ mdhost_lookup_internal(const char *hostname)
 	struct gfarm_hash_entry *entry;
 	static const char diag[] = "mdhost_lookup_internal";
 
-	mdhost_global_mutex_lock(diag);
+	mdhost_table_rwlock_rdlock(diag);
 	entry = gfarm_hash_lookup(mdhost_hashtab, &hostname,
 		sizeof(hostname));
-	mdhost_global_mutex_unlock(diag);
+	mdhost_table_rwlock_unlock(diag);
 	if (entry == NULL)
 		return (NULL);
 	return (*(struct mdhost **)gfarm_hash_entry_data(entry));
@@ -564,7 +572,7 @@ mdhost_lookup_metadb_server(struct gfarm_metadb_server *ms)
 	struct mdhost *m, *mm = NULL;
 	static const char diag[] = "mdhost_lookup_metadb_server";
 
-	mdhost_global_mutex_lock(diag);
+	mdhost_table_rwlock_rdlock(diag);
 	FOREACH_MDHOST(it) {
 		m = mdhost_iterator_access(&it);
 		if (mdhost_is_valid(m) && &m->ms == ms) {
@@ -572,7 +580,7 @@ mdhost_lookup_metadb_server(struct gfarm_metadb_server *ms)
 			break;
 		}
 	}
-	mdhost_global_mutex_unlock(diag);
+	mdhost_table_rwlock_unlock(diag);
 	return (mm);
 }
 
@@ -586,7 +594,7 @@ mdhost_lookup_master(void)
 	m = mdhost_master;
 	mdhost_master_mutex_unlock(diag);
 	if (m == NULL)
-		gflog_fatal(GFARM_MSG_1003324, "%s: no master, abort", diag);
+		gflog_notice(GFARM_MSG_1004732, "%s: no master", diag);
 	return (m);
 }
 
@@ -602,13 +610,33 @@ mdhost_self_is_master(void)
 {
 	struct mdhost *m = mdhost_lookup_self();
 
+	if (m == NULL) {
+		gflog_notice(GFARM_MSG_1004733, "self is not initialized");
+		return (0);
+	}
 	return (mdhost_is_master(m));
 }
 
 /*
- * PREREQUISITE: giant_lock
+ * PREREQUISITE: nothing
+ * LOCKS:
+ *  - abstract_host_mutex
+ *    in abstract_host_disconnect_request()
+ *  - nothing
+ *    in mdhost_unset_peer()
+ *    which is called from abstract_host_peer_unset()
+ *    which is called from abstract_host_disconnect_request()
+ *  - nothing
+ *    in abstract_host::ops::disable() == mdhost_disable()
+ *    which is called from abstract_host_disconnect_request()
+ *  - nothing
+ *    in abstract_host::ops::disabled() == mdhost_disabled()
+ *    which is called from abstract_host_disconnect_request()
+ *  - peer_closing_queue.mutex
+ *    in peer_del_ref()
+ *    which is called from abstract_host_disconnect_request()
  *
- * peer may be NULL.
+ * NOTE: peer may be NULL.
  */
 void
 mdhost_disconnect_request(struct mdhost *m, struct peer *peer)
@@ -618,6 +646,28 @@ mdhost_disconnect_request(struct mdhost *m, struct peer *peer)
 	return (abstract_host_disconnect_request(&m->ah, peer, diag));
 }
 
+/*
+ * PREREQUISITE: nothing
+ * LOCKS:
+ *  - mdhost_master_mutex
+ *    in mdhost_lookup_master()
+ *  - see LOCKS in mdhost_disconnect_request()
+ *
+ * NOTE: peer may be NULL.
+ */
+void
+mdhost_master_disconnect_request(struct peer *peer)
+{
+	struct mdhost *m = mdhost_lookup_master();
+	static const char diag[] = "mdhost_master_disconnect_request";
+
+	if (m == NULL) {
+		gflog_notice(GFARM_MSG_1004749, "%s: no master", diag);
+		return;
+	}
+	mdhost_disconnect_request(m, peer);
+}
+
 /* PREREQUISITE: giant_lock */
 void
 mdhost_set_self_as_master(void)
@@ -645,6 +695,7 @@ mdhost_enter(struct gfarm_metadb_server *ms, struct mdhost **mpp)
 	int created;
 	struct mdhost *mh;
 	gfarm_error_t e;
+	static const char diag[] = "mdhost_enter";
 
 	mh = mdhost_lookup_internal(ms->name);
 	if (mh) {
@@ -678,9 +729,11 @@ mdhost_enter(struct gfarm_metadb_server *ms, struct mdhost **mpp)
 		free(mh);
 		return (e);
 	}
+	mdhost_table_rwlock_wrlock(diag);
 	entry = gfarm_hash_enter(mdhost_hashtab,
 	    &mh->ms.name, sizeof(mh->ms.name),
 	    sizeof(struct mdhost *), &created);
+	mdhost_table_rwlock_unlock(diag);
 	if (entry == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
 		gflog_error(GFARM_MSG_1002930,
@@ -706,6 +759,8 @@ mdhost_is_sync_replication(struct mdhost *mh)
 {
 	struct mdhost *mmh = mdhost_lookup_master();
 
+	if (mmh == NULL)
+		gflog_fatal(GFARM_MSG_1004734, "no master, abort");
 	assert(mh != mmh);
 	return (mh->cluster == mmh->cluster);
 }
@@ -725,9 +780,9 @@ mdhost_has_async_replication_target(void)
 	static const char diag[] = "mdhost_has_async_replication_target";
 	int ret = 0;
 
-	if (mdhost_get_count() == 1)
+	if (mmh == NULL || mdhost_get_count() == 1)
 		return (ret);
-	mdhost_global_mutex_lock(diag);
+	mdhost_table_rwlock_rdlock(diag);
 	FOREACH_MDHOST(it) {
 		mh = mdhost_iterator_access(&it);
 		if (mdhost_is_valid(mh) && mh != mmh &&
@@ -736,7 +791,7 @@ mdhost_has_async_replication_target(void)
 			break;
 		}
 	}
-	mdhost_global_mutex_unlock(diag);
+	mdhost_table_rwlock_unlock(diag);
 	return (ret);
 }
 
@@ -776,7 +831,7 @@ mdhost_update_replication_type(struct mdhost *mh,
 		 * The cluster name of the master has been changed.
 		 * It may cause changes of the replication types of slaves.
 		 */
-		mdhost_global_mutex_lock(diag);
+		mdhost_table_rwlock_rdlock(diag);
 		array_len = 0;
 		FOREACH_MDHOST(it) {
 			if (mdhost_is_valid(mdhost_iterator_access(&it)))
@@ -787,7 +842,7 @@ mdhost_update_replication_type(struct mdhost *mh,
 			e = GFARM_ERR_NO_MEMORY;
 			gflog_error(GFARM_MSG_1004225, "%s",
 			    gfarm_error_string(e));
-			mdhost_global_mutex_unlock(diag);
+			mdhost_table_rwlock_unlock(diag);
 			return (e);
 		}
 		GFARM_MALLOC_ARRAY(to_async_array, array_len);
@@ -796,7 +851,7 @@ mdhost_update_replication_type(struct mdhost *mh,
 			gflog_error(GFARM_MSG_1004226, "%s",
 			    gfarm_error_string(e));
 			free(to_sync_array);
-			mdhost_global_mutex_unlock(diag);
+			mdhost_table_rwlock_unlock(diag);
 			return (e);
 		}
 
@@ -813,7 +868,7 @@ mdhost_update_replication_type(struct mdhost *mh,
 			else if (was_sync && !is_sync)
 				to_async_array[to_async_len++] = m;
 		}
-		mdhost_global_mutex_unlock(diag);
+		mdhost_table_rwlock_unlock(diag);
 
 		if (mdhost_switch_to_sync_hook != NULL) {
 			for (i = 0; i < to_sync_len; i++) {
@@ -842,6 +897,8 @@ mdhost_update_replication_type(struct mdhost *mh,
 		 * Also the replication type of the slave may be changed.
 		 */
 		m = mdhost_lookup_master();
+		if (m == NULL)
+			gflog_fatal(GFARM_MSG_1004735, "no master, abort");
 		mdhost_mutex_lock(m, diag);
 		was_sync = (strcmp(m->ms.clustername, old_clustername) == 0);
 		is_sync  = (strcmp(m->ms.clustername, new_clustername) == 0);
@@ -971,12 +1028,12 @@ mdhost_get_count(void)
 	int n = 0;
 	static const char diag[] = "mdhost_get_count";
 
-	mdhost_global_mutex_lock(diag);
+	mdhost_table_rwlock_rdlock(diag);
 	FOREACH_MDHOST(it) {
 		if (mdhost_is_valid(mdhost_iterator_access(&it)))
 			++n;
 	}
-	mdhost_global_mutex_unlock(diag);
+	mdhost_table_rwlock_unlock(diag);
 	return (n);
 }
 
@@ -1002,7 +1059,7 @@ metadb_server_get0(struct peer *peer, int (*match_op)(
 		    "%s", gfarm_error_string(e));
 	} else {
 		i = 0;
-		mdhost_global_mutex_lock(diag);
+		mdhost_table_rwlock_rdlock(diag);
 		FOREACH_MDHOST(it) {
 			if (i >= nhosts) /* always false due to giant_lock */
 				break;
@@ -1010,7 +1067,7 @@ metadb_server_get0(struct peer *peer, int (*match_op)(
 			if (mdhost_is_valid(mh) && match_op(mh, closure))
 				match[i++] = mh;
 		}
-		mdhost_global_mutex_unlock(diag);
+		mdhost_table_rwlock_unlock(diag);
 		nmatch = i;
 		e = GFARM_ERR_NO_ERROR;
 	}
@@ -1169,7 +1226,7 @@ mdhost_fix_default_master(struct mdhost *new_mmh, const char *diag)
 	struct gfarm_hash_iterator it;
 	struct mdhost *mh;
 
-	mdhost_global_mutex_lock(diag);
+	mdhost_table_rwlock_rdlock(diag);
 	FOREACH_MDHOST(it) {
 		mh = mdhost_iterator_access(&it);
 		if (!mdhost_is_valid(mh))
@@ -1184,7 +1241,7 @@ mdhost_fix_default_master(struct mdhost *new_mmh, const char *diag)
 			break;
 		}
 	}
-	mdhost_global_mutex_unlock(diag);
+	mdhost_table_rwlock_unlock(diag);
 	return (e);
 }
 
@@ -1202,25 +1259,29 @@ mdhost_updated(void)
 	GFARM_MALLOC_ARRAY(mss, n);
 	if (mss == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
-		gflog_debug(GFARM_MSG_1002945,
+		gflog_error(GFARM_MSG_1002945,
 		    "%s: %s", diag, gfarm_error_string(e));
 		return (e);
 	}
 	i = 0;
-	mdhost_global_mutex_lock(diag);
+	mdhost_table_rwlock_rdlock(diag);
 	FOREACH_MDHOST(it) {
 		mh = mdhost_iterator_access(&it);
 		if (mdhost_is_valid(mh) && i < n)
 			mss[i++] = &mh->ms;
 	}
-	mdhost_global_mutex_unlock(diag);
+	mdhost_table_rwlock_unlock(diag);
 	fs = gfarm_filesystem_get_default();
-	gfarm_filesystem_set_metadb_server_list(fs, mss, i);
+	e = gfarm_filesystem_set_metadb_server_list(fs, mss, i);
+	if (e != GFARM_ERR_NO_ERROR)
+		gflog_error(GFARM_MSG_1004730,
+		    "%s: gfarm_filesystem_set_metadb_server_list: %s",
+		    diag, gfarm_error_string(e));
 	free(mss);
 	if (mdhost_update_hook_for_journal_send != NULL)
 		(*mdhost_update_hook_for_journal_send)();
 
-	return (GFARM_ERR_NO_ERROR);
+	return (e);
 }
 
 static gfarm_error_t
@@ -1348,9 +1409,9 @@ gfm_server_metadb_server_set(struct peer *peer, int from_client, int skip)
 		ms.name = ms.clustername = NULL;
 	}
 
-	if (e == GFARM_ERR_NO_ERROR)
-		mdhost_updated();
-	else {
+	if (e == GFARM_ERR_NO_ERROR) {
+		e = mdhost_updated();
+	} else {
 		gflog_debug(GFARM_MSG_1002959,
 		    "error occurred during process: %s",
 		    gfarm_error_string(e));
@@ -1419,7 +1480,7 @@ gfm_server_metadb_server_modify(struct peer *peer, int from_client, int skip)
 				    gfarm_error_string(e));
 		}
 		if (e == GFARM_ERR_NO_ERROR)
-			mdhost_updated();
+			e = mdhost_updated();
 	}
 	giant_unlock();
 	gfarm_metadb_server_free(&ms);
@@ -1480,13 +1541,7 @@ gfm_server_metadb_server_remove(struct peer *peer, int from_client, int skip)
 void
 mdhost_init(void)
 {
-	struct mdhost *self;
-	struct gfarm_metadb_server ms;
 	gfarm_error_t e;
-	struct mdhost *mh;
-	struct gfarm_hash_iterator it;
-	static const char diag[] = "mdhost_init";
-	char *metadb_server_name = gfarm_ctxp->metadb_server_name;
 
 	if (gfarm_get_metadb_replication_enabled())
 		mdcluster_init();
@@ -1504,6 +1559,19 @@ mdhost_init(void)
 			gflog_fatal(GFARM_MSG_1002971,
 			    "%s", gfarm_error_string(e));
 	}
+}
+
+void
+mdhost_initial_entry(void)
+{
+	struct mdhost *self;
+	struct gfarm_metadb_server ms;
+	gfarm_error_t e;
+	struct mdhost *mh;
+	struct gfarm_hash_iterator it;
+	static const char diag[] = "mdhost_initial_entry";
+	char *metadb_server_name = gfarm_ctxp->metadb_server_name;
+
 	if ((self = mdhost_lookup(metadb_server_name)) == NULL) {
 		ms.name = strdup_ck(metadb_server_name, diag);
 		ms.port = gfmd_port;
@@ -1529,6 +1597,7 @@ mdhost_init(void)
 	mdhost_self = self;
 	mdhost_set_is_self(mdhost_self, 1);
 	if (gfarm_get_metadb_replication_enabled()) {
+		mdhost_table_rwlock_rdlock(diag);
 		FOREACH_MDHOST(it) {
 			mh = mdhost_iterator_access(&it);
 			if (!mdhost_is_valid(mh))
@@ -1538,6 +1607,7 @@ mdhost_init(void)
 				break;
 			}
 		}
+		mdhost_table_rwlock_unlock(diag);
 		if ((e = mdhost_updated()) != GFARM_ERR_NO_ERROR)
 			gflog_fatal(GFARM_MSG_1002975,
 			    "Failed to update mdhost: %s",
