@@ -4,6 +4,7 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <time.h>
 
 #define GFARM_INTERNAL_USE
@@ -65,13 +66,13 @@ static struct gfarm_id_table_entry_ops process_id_table_ops = {
 	sizeof(struct process)
 };
 
-struct replication_info {
+struct client_initiated_replication {
 	gfarm_int64_t gen;
 	struct host *dst;
 	gfarm_int32_t cksum_request_flags;
 };
 
-static struct file_opening *
+struct file_opening *
 file_opening_alloc(struct inode *inode,
 	struct peer *peer, struct host *spool_host, int flag)
 {
@@ -86,6 +87,7 @@ file_opening_alloc(struct inode *inode,
 	fo->inode = inode;
 	fo->flag = flag;
 	fo->opener = peer;
+	fo->gen = inode_get_gen(inode);
 
 	if (inode_is_file(inode)) {
 		if (spool_host == NULL) {
@@ -692,7 +694,7 @@ process_new_generation_done(struct process *process, struct peer *peer, int fd,
 gfarm_error_t
 process_open_file(struct process *process, struct inode *file,
 	gfarm_int32_t flag, int created,
-	struct peer *peer, struct host *spool_host,
+	struct peer *peer, struct host *spool_host, struct dirset *tdirset,
 	gfarm_int32_t *fdp)
 {
 	gfarm_error_t e;
@@ -731,7 +733,7 @@ process_open_file(struct process *process, struct inode *file,
 			"file_opening_alloc() failed");
 		return (GFARM_ERR_NO_MEMORY);
 	}
-	e = inode_open(fo);
+	e = inode_open(fo, tdirset);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001626,
 			"inode_open() failed: %s",
@@ -906,6 +908,23 @@ process_peer_is_the_spool_opener(struct process *process,
 	return (1);
 }
 
+gfarm_error_t
+process_getgen(struct process *process, struct peer *peer, int fd,
+	gfarm_uint64_t *genp, const char *diag)
+{
+	struct file_opening *fo;
+	gfarm_error_t e = process_get_file_opening(process, peer, fd,
+	    &fo, diag);
+
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1004701,
+			"process_get_file_opening() failed: %s",
+			gfarm_error_string(e));
+		return (e);
+	}
+	*genp = fo->gen;
+	return (GFARM_ERR_NO_ERROR);
+}
 static gfarm_error_t
 process_close_or_abort_file(struct process *process, struct peer *peer, int fd,
 	char **trace_logp, int aborted, const char *diag)
@@ -1083,10 +1102,8 @@ process_close_file_write(struct process *process, struct peer *peer, int fd,
 	    ((fo->flag & GFARM_FILE_CREATE_REPLICA) == 0 ||
 	    inode_add_replica(fo->inode, fo->u.f.spool_host, 1)
 	    == GFARM_ERR_ALREADY_EXISTS) &&
-
 	    inode_file_update(fo, size, atime, mtime, is_v2_4,
 	    old_genp, new_genp, trace_logp, diag)) {
-
 		flags = GFM_PROTO_CLOSE_WRITE_GENERATION_UPDATE_NEEDED;
 	}
 
@@ -1220,7 +1237,8 @@ process_inherit_fd(struct process *process, gfarm_int32_t parent_fd,
 		return (GFARM_ERR_BAD_FILE_DESCRIPTOR);
 	}
 	return (process_open_file(process, fo->inode, fo->flag,
-	    (fo->flag & GFARM_FILE_CREATE) != 0, peer, spool_host, fdp));
+	    (fo->flag & GFARM_FILE_CREATE) != 0, peer, spool_host,
+	    inode_get_tdirset(fo->inode), fdp));
 }
 
 static gfarm_error_t
