@@ -1,32 +1,39 @@
 /*
- * $Id: quota.c 10227 2016-09-27 12:21:25Z tatebe $
+ * $Id: quota.c 10660 2017-11-08 00:58:37Z n-soda $
  */
 
 #include <pthread.h>	/* db_access.h currently needs this */
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <time.h>
 #include <sys/time.h>
 #include <assert.h>
 
 #include <gfarm/gfarm.h>
 
+#include "gfutil.h"
 #include "thrsubr.h"
 
 #include "auth.h"
 #include "quota_info.h"
+#include "config.h"
 
+#include "uint64_map.h"
 #include "peer.h"
 #include "subr.h"
 #include "rpcsubr.h"
 #include "user.h"
 #include "group.h"
 #include "inode.h"
+#include "dir.h"
 #include "quota.h"
+#include "dirset.h"
+#include "quota_dir.h"
 #include "db_access.h"
 
 #define QUOTA_NOT_CHECK_YET -1
-#define is_checked(q) (q->space != QUOTA_NOT_CHECK_YET ? 1 : 0)
+#define is_checked(q) ((q)->space != QUOTA_NOT_CHECK_YET)
 
 static gfarm_error_t db_state = GFARM_ERR_NO_ERROR;
 
@@ -83,6 +90,42 @@ quota_softlimit_exceed(struct quota *q, int *need_db_update)
 			 q->phy_num, q->phy_num_soft, need_db_update);
 }
 
+static void
+quota_metadata_softlimit_exceed(struct quota_metadata *q, int *need_db_update)
+{
+	struct timeval now;
+
+	if (!quota_limit_is_valid(q->limit.grace_period)) {
+		if (q->exceed.space_time != GFARM_QUOTA_INVALID ||
+		    q->exceed.num_time != GFARM_QUOTA_INVALID ||
+		    q->exceed.phy_space_time != GFARM_QUOTA_INVALID ||
+		    q->exceed.phy_num_time != GFARM_QUOTA_INVALID) {
+			/* disable all softlimit */
+			q->exceed.space_time = GFARM_QUOTA_INVALID;
+			q->exceed.num_time = GFARM_QUOTA_INVALID;
+			q->exceed.phy_space_time = GFARM_QUOTA_INVALID;
+			q->exceed.phy_num_time = GFARM_QUOTA_INVALID;
+			*need_db_update = 1;
+		}
+		return;
+	}
+
+	/* update exceeded time of softlimit */
+	gettimeofday(&now, NULL);
+	update_softlimit(&q->exceed.space_time, now.tv_sec,
+	    q->limit.grace_period, q->usage.space,
+	    q->limit.soft.space, need_db_update);
+	update_softlimit(&q->exceed.num_time, now.tv_sec,
+	    q->limit.grace_period, q->usage.num,
+	    q->limit.soft.num, need_db_update);
+	update_softlimit(&q->exceed.phy_space_time, now.tv_sec,
+	    q->limit.grace_period, q->usage.phy_space,
+	    q->limit.soft.phy_space, need_db_update);
+	update_softlimit(&q->exceed.phy_num_time, now.tv_sec,
+	    q->limit.grace_period, q->usage.phy_num,
+	    q->limit.soft.phy_num, need_db_update);
+}
+
 static void
 quota_softlimit_exceed_user(struct quota *q, struct user *u)
 {
@@ -115,8 +158,26 @@ quota_softlimit_exceed_group(struct quota *q, struct group *g)
 	}
 }
 
+void
+dirquota_softlimit_exceed(struct quota_metadata *q, struct dirset *ds)
+{
+	gfarm_error_t e;
+	int need_db_update = 0;
+
+	quota_metadata_softlimit_exceed(q, &need_db_update);
+	if (need_db_update && dirset_is_valid(ds)) {
+		e = db_quota_dirset_modify(
+		    dirset_get_username(ds), dirset_get_dirsetname(ds), q);
+		if (e != GFARM_ERR_NO_ERROR)
+			gflog_error(GFARM_MSG_1004634,
+			    "db_quota_dirset_modify(%s:%s): %s",
+			    dirset_get_username(ds), dirset_get_dirsetname(ds),
+			    gfarm_error_string(e));
+	}
+}
+
 static void
-usage_clear(struct usage *usage)
+quota_usage_clear(struct gfarm_quota_subject_info *usage)
 {
 	usage->space = 0;
 	usage->num = 0;
@@ -127,38 +188,38 @@ usage_clear(struct usage *usage)
 static void
 usage_tmp_clear_user(void *closure, struct user *u)
 {
-	usage_clear(user_usage_tmp(u));
+	quota_usage_clear(user_usage_tmp(u));
 }
 
 static void
 usage_tmp_clear_group(void *closure, struct group *g)
 {
-	usage_clear(group_usage_tmp(g));
+	quota_usage_clear(group_usage_tmp(g));
 }
 
 static void
 usage_tmp_clear(void)
 {
-	user_all(NULL, usage_tmp_clear_user, 0);
-	group_all(NULL, usage_tmp_clear_group, 0);
+	user_foreach(NULL, usage_tmp_clear_user, 0);
+	group_foreach(NULL, usage_tmp_clear_group, 0);
 }
 
 static void
-usage_to_quota(struct usage *src, struct quota *dst)
+usage_to_quota(struct gfarm_quota_subject_info *src_usage, struct quota *dst)
 {
-	dst->space = src->space;
-	dst->num = src->num;
-	dst->phy_space = src->phy_space;
-	dst->phy_num = src->phy_num;
+	dst->space = src_usage->space;
+	dst->num = src_usage->num;
+	dst->phy_space = src_usage->phy_space;
+	dst->phy_num = src_usage->phy_num;
 }
 
 static void
 quota_update_usage_user(void *closure, struct user *u)
 {
 	struct quota *q = user_quota(u);
-	struct usage *ut = user_usage_tmp(u);
+	struct gfarm_quota_subject_info *usage_tmp = user_usage_tmp(u);
 
-	usage_to_quota(ut, q);
+	usage_to_quota(usage_tmp, q);
 	quota_softlimit_exceed_user(q, u);
 
 	if (!user_is_valid(u))
@@ -174,9 +235,9 @@ static void
 quota_update_usage_group(void *closure, struct group *g)
 {
 	struct quota *q = group_quota(g);
-	struct usage *ut = group_usage_tmp(g);
+	struct gfarm_quota_subject_info *usage_tmp = group_usage_tmp(g);
 
-	usage_to_quota(ut, q);
+	usage_to_quota(usage_tmp, q);
 	quota_softlimit_exceed_group(q, g);
 
 	if (!group_is_valid(g))
@@ -189,10 +250,10 @@ quota_update_usage_group(void *closure, struct group *g)
 }
 
 static void
-quota_update_usage()
+quota_update_usage(void)
 {
-	user_all(NULL, quota_update_usage_user, 0);
-	group_all(NULL, quota_update_usage_group, 0);
+	user_foreach(NULL, quota_update_usage_user, 0);
+	group_foreach(NULL, quota_update_usage_group, 0);
 }
 
 static gfarm_time_t
@@ -213,6 +274,25 @@ calculate_grace_period(gfarm_time_t exceeded_time,
 	return (val); /* grace period until expiration */
 }
 
+void
+quota_exceed_to_grace(gfarm_time_t grace_period,
+	const struct gfarm_quota_subject_time *exceed,
+	struct gfarm_quota_subject_time *grace)
+{
+	struct timeval now;
+
+	gettimeofday(&now, NULL);
+
+	grace->space_time = calculate_grace_period(
+	    exceed->space_time, grace_period, now.tv_sec);
+	grace->num_time = calculate_grace_period(
+	    exceed->num_time, grace_period, now.tv_sec);
+	grace->phy_space_time = calculate_grace_period(
+	    exceed->phy_space_time, grace_period, now.tv_sec);
+	grace->phy_num_time = calculate_grace_period(
+	    exceed->phy_num_time, grace_period, now.tv_sec);
+}
+
 static void
 quota_convert_1(const struct quota *q, const char *name,
 		struct gfarm_quota_get_info *qi)
@@ -327,7 +407,7 @@ quota_group_set_one_from_db(void *closure, struct gfarm_quota_info *qi)
 }
 
 void
-quota_init()
+quota_init(void)
 {
 	gfarm_error_t e;
 
@@ -371,6 +451,110 @@ quota_data_init(struct quota *q)
 	q->phy_num_hard = GFARM_QUOTA_INVALID;
 }
 
+/* for soft and hard */
+static void
+quota_limit_subject_init(struct gfarm_quota_subject_info *limit)
+{
+	limit->space = GFARM_QUOTA_INVALID;
+	limit->num = GFARM_QUOTA_INVALID;
+	limit->phy_space = GFARM_QUOTA_INVALID;
+	limit->phy_num = GFARM_QUOTA_INVALID;
+}
+
+/* for limit */
+static void
+quota_limit_init(struct gfarm_quota_limit_info *limit)
+{
+	limit->grace_period = GFARM_QUOTA_INVALID; /* disable all softlimit */
+	quota_limit_subject_init(&limit->soft);
+	quota_limit_subject_init(&limit->hard);
+}
+
+/* for exceed, grace */
+void
+quota_subject_time_init(struct gfarm_quota_subject_time *time)
+{
+	time->space_time = GFARM_QUOTA_INVALID;
+	time->num_time = GFARM_QUOTA_INVALID;
+	time->phy_space_time = GFARM_QUOTA_INVALID;
+	time->phy_num_time = GFARM_QUOTA_INVALID;
+}
+
+void
+quota_metadata_init(struct quota_metadata *q)
+{
+	quota_limit_init(&q->limit);
+	quota_usage_clear(&q->usage);
+	quota_subject_time_init(&q->exceed);
+}
+
+static void
+quota_metadata_memory_init(struct quota_metadata_memory *qmm)
+{
+	quota_metadata_init(&qmm->q);
+	qmm->usage_is_valid = 0;
+}
+
+void
+quota_metadata_memory_convert_to_db(
+	const struct quota_metadata_memory *qmm, struct quota_metadata *q)
+{
+	*q = qmm->q;
+	if (!qmm->usage_is_valid) {
+		quota_usage_clear(&q->usage);
+		q->usage.space = QUOTA_NOT_CHECK_YET;
+	}
+}
+
+void
+quota_metadata_memory_convert_from_db(
+	struct quota_metadata_memory *qmm, const struct quota_metadata *q)
+{
+	qmm->q = *q;
+
+#if 0
+	qmm->usage_is_valid = is_checked(&q->usage);
+	if (!qmm->usage_is_valid)
+		quota_usage_clear(&qmm->q.usage);
+#else /* usage in backend DB is garbage */
+	quota_usage_clear(&qmm->q.usage);
+	qmm->usage_is_valid = 0;
+#endif
+}
+
+void
+dirquota_init(struct dirquota *dq)
+{
+	quota_metadata_memory_init(&dq->qmm);
+
+	/*
+	 * at the first place, usage is zero inodes and zero bytes.
+	 * and then, if quota_metadata_memory_convert_from_db() is called via
+	 * dirset_set_quota_metadata_in_cache(), usage_is_valid becomes 0.
+	 */
+	dq->qmm.usage_is_valid = 1;
+
+	dq->dirquota_checking = 0;
+	dq->invalidate_requested = 0;
+}
+
+/* this is protected by giant_lock */
+static int dirquota_invalidate_all_requested = 0;
+
+int
+dirquota_is_checked(const struct dirquota *dq)
+{
+	return (!dirquota_invalidate_all_requested &&
+	    dq->qmm.usage_is_valid && !dq->invalidate_requested);
+}
+
+void
+dirquota_check_retry_if_running(struct dirquota *dq)
+{
+	if (dq->dirquota_checking)
+		dq->invalidate_requested = 1;
+}
+
 static inline gfarm_int64_t
 int64_add(gfarm_int64_t orig, gfarm_int64_t diff)
 {
@@ -398,10 +582,10 @@ int64_add(gfarm_int64_t orig, gfarm_int64_t diff)
 
 #define update_file_add(q, size, ncopy)					\
 	{								\
-		q->space = int64_add(q->space, size);			\
-		q->num = int64_add(q->num, 1);				\
-		q->phy_space = int64_add(q->phy_space, size * ncopy);	\
-		q->phy_num = int64_add(q->phy_num, ncopy);		\
+		(q)->space = int64_add((q)->space, size);		\
+		(q)->num = int64_add((q)->num, 1);			\
+		(q)->phy_space = int64_add((q)->phy_space, size * ncopy); \
+		(q)->phy_num = int64_add((q)->phy_num, ncopy);		\
 	}
 
 
@@ -427,10 +611,28 @@ usage_tmp_update(struct inode *inode)
 		update_file_add(group_usage_tmp(g), size, ncopy);
 }
 
+static void
+dirquota_usage_tmp_update(struct dirset *ds, struct inode *inode)
+{
+	gfarm_off_t size;
+	gfarm_int64_t ncopy;
+	struct dirquota *dq = dirset_get_dirquota(ds);
+
+	if (inode_is_file(inode)) {
+		size = inode_get_size(inode);
+		ncopy = inode_get_ncopy_with_dead_host(inode);
+	} else {
+		size = 0;
+		ncopy = 0;
+	}
+
+	update_file_add(&dq->usage_tmp, size, ncopy);
+}
+
 static void quota_check_retry_if_running(void);
 
 void
-quota_update_file_add(struct inode *inode)
+quota_update_file_add(struct inode *inode, struct dirset *tdirset)
 {
 	gfarm_off_t size;
 	gfarm_int64_t ncopy;
@@ -459,6 +661,13 @@ quota_update_file_add(struct inode *inode)
 			quota_softlimit_exceed_group(gq, g);
 		}
 	}
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET) {
+		struct dirquota *dq = dirset_get_dirquota(tdirset);
+		update_file_add(&dq->qmm.q.usage, size, ncopy);
+		dirquota_softlimit_exceed(&dq->qmm.q, tdirset);
+		if (!dirquota_is_checked(dq))
+			dirquota_check_retry_if_running(dq);
+	}
 
 	if (debug_mode) {
 		gfarm_ino_t inum;
@@ -482,12 +691,13 @@ quota_update_file_add(struct inode *inode)
 #define update_file_resize(q, old_size, new_size, ncopy)		\
 	{								\
 		gfarm_int64_t diff = new_size - old_size;		\
-		q->space = int64_add(q->space, diff);			\
-		q->phy_space = int64_add(q->phy_space, diff * ncopy);	\
+		(q)->space = int64_add((q)->space, diff);		\
+		(q)->phy_space = int64_add((q)->phy_space, diff * ncopy); \
 	}
 
 void
-quota_update_file_resize(struct inode *inode, gfarm_off_t new_size)
+quota_update_file_resize(struct inode *inode, struct dirset *tdirset,
+	gfarm_off_t new_size)
 {
 	gfarm_off_t old_size = inode_get_size(inode);
 	gfarm_int64_t ncopy = inode_get_ncopy_with_dead_host(inode);
@@ -510,18 +720,27 @@ quota_update_file_resize(struct inode *inode, gfarm_off_t new_size)
 			quota_softlimit_exceed_group(gq, g);
 		}
 	}
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET) {
+		struct dirquota *dq = dirset_get_dirquota(tdirset);
+		update_file_resize(&dq->qmm.q.usage,
+		    old_size, new_size, ncopy);
+		dirquota_softlimit_exceed(&dq->qmm.q, tdirset);
+		if (!dirquota_is_checked(dq))
+			dirquota_check_retry_if_running(dq);
+	}
 
 	quota_check_retry_if_running();
 }
 
 #define update_replica_num(q, size, n)					\
 	{								\
-		q->phy_space = int64_add(q->phy_space, size * n);	\
-		q->phy_num = int64_add(q->phy_num, n);			\
+		(q)->phy_space = int64_add((q)->phy_space, size * n);	\
+		(q)->phy_num = int64_add((q)->phy_num, n);		\
 	}
 
 static void
-quota_update_replica_num(struct inode *inode, gfarm_int64_t n)
+quota_update_replica_num(struct inode *inode, struct dirset *tdirset,
+	gfarm_int64_t n)
 {
 	gfarm_off_t size = inode_get_size(inode);
 	struct user *u = inode_get_user(inode);
@@ -541,32 +760,39 @@ quota_update_replica_num(struct inode *inode, gfarm_int64_t n)
 			quota_softlimit_exceed_group(gq, g);
 		}
 	}
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET) {
+		struct dirquota *dq = dirset_get_dirquota(tdirset);
+		update_replica_num(&dq->qmm.q.usage, size, n);
+		dirquota_softlimit_exceed(&dq->qmm.q, tdirset);
+		if (!dirquota_is_checked(dq))
+			dirquota_check_retry_if_running(dq);
+	}
 
 	quota_check_retry_if_running();
 }
 
 void
-quota_update_replica_add(struct inode *inode)
+quota_update_replica_add(struct inode *inode, struct dirset *tdirset)
 {
-	quota_update_replica_num(inode, 1);
+	quota_update_replica_num(inode, tdirset, 1);
 }
 
 void
-quota_update_replica_remove(struct inode *inode)
+quota_update_replica_remove(struct inode *inode, struct dirset *tdirset)
 {
-	quota_update_replica_num(inode, -1);
+	quota_update_replica_num(inode, tdirset, -1);
 }
 
 #define update_file_remove(q, size, ncopy)				\
 	{								\
-		q->space = int64_add(q->space, -size);			\
-		q->num = int64_add(q->num, -1);				\
-		q->phy_space = int64_add(q->phy_space, -(size * ncopy)); \
-		q->phy_num = int64_add(q->phy_num, -ncopy);		\
+		(q)->space = int64_add((q)->space, -size);		\
+		(q)->num = int64_add((q)->num, -1);			\
+		(q)->phy_space = int64_add((q)->phy_space, -(size * ncopy)); \
+		(q)->phy_num = int64_add((q)->phy_num, -ncopy);		\
 	}
 
 void
-quota_update_file_remove(struct inode *inode)
+quota_update_file_remove(struct inode *inode, struct dirset *tdirset)
 {
 	gfarm_off_t size;
 	gfarm_int64_t ncopy;
@@ -595,10 +821,67 @@ quota_update_file_remove(struct inode *inode)
 			quota_softlimit_exceed_group(gq, g);
 		}
 	}
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET) {
+		struct dirquota *dq = dirset_get_dirquota(tdirset);
+		update_file_remove(&dq->qmm.q.usage, size, ncopy);
+		dirquota_softlimit_exceed(&dq->qmm.q, tdirset);
+		if (!dirquota_is_checked(dq))
+			dirquota_check_retry_if_running(dq);
+	}
 
 	quota_check_retry_if_running();
 }
 
+void
+dirquota_update_file_add(struct inode *inode, struct dirset *tdirset)
+{
+	gfarm_off_t size;
+	gfarm_int64_t ncopy;
+
+	if (inode_is_file(inode)) {
+		size = inode_get_size(inode);
+		ncopy = inode_get_ncopy_with_dead_host(inode);
+	} else {
+		size = 0;
+		ncopy = 0;
+	}
+
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET) {
+		struct dirquota *dq = dirset_get_dirquota(tdirset);
+		update_file_add(&dq->qmm.q.usage, size, ncopy);
+		dirquota_softlimit_exceed(&dq->qmm.q, tdirset);
+		if (!dirquota_is_checked(dq))
+			dirquota_check_retry_if_running(dq);
+	}
+
+	/* quota_check_retry_if_running() is unnecessary here */
+}
+
+void
+dirquota_update_file_remove(struct inode *inode, struct dirset *tdirset)
+{
+	gfarm_off_t size;
+	gfarm_int64_t ncopy;
+
+	if (inode_is_file(inode)) {
+		size = inode_get_size(inode);
+		ncopy = inode_get_ncopy_with_dead_host(inode);
+	} else {
+		size = 0;
+		ncopy = 0;
+	}
+
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET) {
+		struct dirquota *dq = dirset_get_dirquota(tdirset);
+		update_file_remove(&dq->qmm.q.usage, size, ncopy);
+		dirquota_softlimit_exceed(&dq->qmm.q, tdirset);
+		if (!dirquota_is_checked(dq))
+			dirquota_check_retry_if_running(dq);
+	}
+
+	/* quota_check_retry_if_running() is unnecessary here */
+}
+
 enum quota_exceeded_type {
 	QUOTA_NOT_EXCEEDED = 0,
 	QUOTA_EXCEEDED_SPACE_SOFT,
@@ -671,6 +954,75 @@ is_exceeded(struct timeval *nowp, struct quota *q,
 	return (QUOTA_NOT_EXCEEDED);
 }
 
+static int
+dirquota_is_exceeded(struct timeval *nowp, struct dirquota *dq,
+	int num_file_creating, int num_replica_adding, gfarm_off_t size)
+{
+	int check_logical = 0, check_physical = 0;
+	struct quota_metadata *q;
+
+#if 0 /* check dirquota, even if it is inaccurate */
+	if (!dirquota_is_checked(dq))  /* quota is disabled */
+		return (QUOTA_NOT_EXCEEDED);
+#endif
+	q = &dq->qmm.q;
+
+	/* softlimit */
+	if (quota_limit_is_valid(q->limit.grace_period)) {
+		if (quota_limit_is_valid(q->limit.soft.space) &&
+		    quota_limit_is_valid(q->exceed.space_time) &&
+		    (nowp->tv_sec - q->exceed.space_time) >
+		    q->limit.grace_period)
+			return (QUOTA_EXCEEDED_SPACE_SOFT);
+		if (quota_limit_is_valid(q->limit.soft.num) &&
+		    quota_limit_is_valid(q->exceed.num_time) &&
+		    (nowp->tv_sec - q->exceed.num_time) >
+		    q->limit.grace_period)
+			return (QUOTA_EXCEEDED_NUM_SOFT);
+		if (quota_limit_is_valid(q->limit.soft.phy_space) &&
+		    quota_limit_is_valid(q->exceed.phy_space_time) &&
+		    (nowp->tv_sec - q->exceed.phy_space_time) >
+		    q->limit.grace_period)
+			return (QUOTA_EXCEEDED_PHY_SPACE_SOFT);
+		if (quota_limit_is_valid(q->limit.soft.phy_num) &&
+		    quota_limit_is_valid(q->exceed.phy_num_time) &&
+		    (nowp->tv_sec - q->exceed.phy_num_time) >
+		    q->limit.grace_period)
+			return (QUOTA_EXCEEDED_PHY_NUM_SOFT);
+	}
+
+	/* hardlimit */
+	if (num_file_creating >= 1 && num_replica_adding <= 0) {
+		check_logical = 1;
+	} else if (num_file_creating <= 0 && num_replica_adding >= 1) {
+		check_physical = 1;
+	} else {
+		check_logical = 1;
+		check_physical = 1;
+	}
+
+	if (check_logical) {
+		if ((quota_limit_is_valid(q->limit.hard.space) &&
+		    q->usage.space + size > q->limit.hard.space))
+			return (QUOTA_EXCEEDED_SPACE_HARD);
+		if (quota_limit_is_valid(q->limit.hard.num) &&
+		    q->usage.num + num_file_creating > q->limit.hard.num)
+			return (QUOTA_EXCEEDED_NUM_HARD);
+	}
+	if (check_physical) {
+		if (quota_limit_is_valid(q->limit.hard.phy_space) &&
+		    q->usage.phy_space + (size * num_replica_adding) >
+		    q->limit.hard.phy_space)
+			return (QUOTA_EXCEEDED_PHY_SPACE_HARD);
+		if (quota_limit_is_valid(q->limit.hard.phy_num) &&
+		    q->usage.phy_num + num_replica_adding >
+		    q->limit.hard.phy_num)
+			return (QUOTA_EXCEEDED_PHY_NUM_HARD);
+	}
+
+	return (QUOTA_NOT_EXCEEDED);
+}
+
 /*
  * num_file_creating >= 1 && num_replica_adding == 0 : check logical quota
  * num_file_creating == 0 && num_replica_adding >= 1 : check physical quota
@@ -678,7 +1030,7 @@ is_exceeded(struct timeval *nowp, struct quota *q,
  * num_file_creating >= 1 && num_replica_adding >= 1 : check both
  */
 gfarm_error_t
-quota_limit_check(struct user *u, struct group *g,
+quota_limit_check(struct user *u, struct group *g, struct dirset *tdirset,
 	int num_file_creating, int num_replica_adding, gfarm_off_t size)
 {
 	struct timeval now;
@@ -696,6 +1048,35 @@ quota_limit_check(struct user *u, struct group *g,
 			 "group_quota(%s) exceeded", group_name(g));
 		return (GFARM_ERR_DISK_QUOTA_EXCEEDED);
 	}
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET &&
+	    dirquota_is_exceeded(&now, dirset_get_dirquota(tdirset),
+	    num_file_creating, num_replica_adding, size)) {
+		gflog_debug(GFARM_MSG_1004635,
+		    "dirset_quota(%s:%s) exceeded",
+		    dirset_get_username(tdirset),
+		    dirset_get_dirsetname(tdirset));
+		return (GFARM_ERR_DISK_QUOTA_EXCEEDED);
+	}
+
+	return (GFARM_ERR_NO_ERROR);
+}
+
+gfarm_error_t
+dirquota_limit_check(struct dirset *tdirset,
+	int num_file_creating, int num_replica_adding, gfarm_off_t size)
+{
+	struct timeval now;
+
+	gettimeofday(&now, NULL);
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET &&
+	    dirquota_is_exceeded(&now, dirset_get_dirquota(tdirset),
+	    num_file_creating, num_replica_adding, size)) {
+		gflog_debug(GFARM_MSG_1004636,
+		    "dirset_quota(%s:%s) exceeded",
+		    dirset_get_username(tdirset),
+		    dirset_get_dirsetname(tdirset));
+		return (GFARM_ERR_DISK_QUOTA_EXCEEDED);
+	}
 
 	return (GFARM_ERR_NO_ERROR);
 }
@@ -724,33 +1105,145 @@ quota_group_remove(struct group *g)
 	q->space = QUOTA_NOT_CHECK_YET;
 }
 
-static pthread_mutex_t quota_check_mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_cond_t quota_check_wakeup = PTHREAD_COND_INITIALIZER;
-static pthread_cond_t quota_check_end = PTHREAD_COND_INITIALIZER;
-static int quota_check_needed = 0;
-static int quota_check_running = 0;
+/*
+ * common part of quota_check and dirquota_check
+ */
 
-static const char quota_check_mutex_diag[] = "quota_check_mutex";
-static const char quota_check_wakeup_diag[] = "quota_check_wakeup";
-static const char quota_check_end_diag[] = "quota_check_end";
+struct quota_check_control {
+	void (*main_function)(struct quota_check_control *);
+
+	pthread_mutex_t mutex;
+	pthread_cond_t wakeup;
+	pthread_cond_t end;
+	time_t target_time;
+	int needed;
+	int running;
+
+	const char *mutex_diag;
+	const char *wakeup_diag;
+	const char *end_diag;
+};
+
+static void
+quota_check_schedule(struct quota_check_control *ctl, time_t target_time)
+{
+	static const char diag[] = "quota_check_schedule";
+
+	gfarm_mutex_lock(&ctl->mutex, diag, ctl->mutex_diag);
+	ctl->needed = 1;
+	ctl->target_time = target_time;
+	gfarm_cond_signal(&ctl->wakeup, diag, ctl->wakeup_diag);
+	gfarm_mutex_unlock(&ctl->mutex, diag, ctl->mutex_diag);
+}
+
+static void
+quota_check_start(struct quota_check_control *ctl)
+{
+	quota_check_schedule(ctl, time(NULL));
+}
+
+static void
+quota_check_wait_for_end(struct quota_check_control *ctl)
+{
+	static const char diag[] = "quota_check_wait_for_end";
+
+	gfarm_mutex_lock(&ctl->mutex, diag, ctl->mutex_diag);
+	while (ctl->needed || ctl->running)
+		gfarm_cond_wait(&ctl->end, &ctl->mutex, diag, ctl->end_diag);
+	gfarm_mutex_unlock(&ctl->mutex, diag, ctl->mutex_diag);
+}
+
+static void *
+quota_check_thread(void *arg)
+{
+	struct quota_check_control *ctl = arg;
+	time_t t;
+	static const char diag[] = "quota_check_thread";
+
+	(void)gfarm_pthread_set_priority_minimum(diag);
+
+	for (;;) {
+		gfarm_mutex_lock(&ctl->mutex, diag, ctl->mutex_diag);
+		ctl->running = 0;
+		gfarm_cond_signal(&ctl->end, diag, ctl->end_diag);
+
+		while (!ctl->needed)
+			gfarm_cond_wait(&ctl->wakeup, &ctl->mutex,
+			    diag, ctl->wakeup_diag);
+
+		for (;;) {
+			t = time(NULL);
+			if (ctl->target_time <= t)
+				break;
+			t = ctl->target_time - t;
+			gfarm_mutex_unlock(&ctl->mutex, diag, ctl->mutex_diag);
+			gfarm_sleep(t);
+			gfarm_mutex_lock(&ctl->mutex, diag, ctl->mutex_diag);
+		}
+
+		ctl->needed = 0;
+		ctl->running = 1;
+		gfarm_mutex_unlock(&ctl->mutex, diag, ctl->mutex_diag);
+
+		(*ctl->main_function)(ctl);
+	}
+
+	return (NULL);
+}
+
+static gfarm_error_t
+quota_check_at_the_start(struct quota_check_control *ctl)
+{
+	gfarm_error_t e = create_detached_thread(quota_check_thread, ctl);
+
+	if (e != GFARM_ERR_NO_ERROR)
+		return (e);
+
+	quota_check_start(ctl);
+	quota_check_wait_for_end(ctl);
+	return (GFARM_ERR_NO_ERROR);
+}
+
+/*
+ * quota_check
+ */
+
+static void quota_check_main_loop(struct quota_check_control *);
+
+static struct quota_check_control quota_check_ctl = {
+	quota_check_main_loop,
+	PTHREAD_MUTEX_INITIALIZER,
+	PTHREAD_COND_INITIALIZER,
+	PTHREAD_COND_INITIALIZER,
+	0,
+	0,
+	0,
+	"quota_check_mutex",
+	"quota_check_wakeup",
+	"quota_check_end",
+};
 
 static int
 quota_check_needed_locked(const char *diag)
 {
 	int needed;
 
-	gfarm_mutex_lock(&quota_check_mutex, diag, quota_check_mutex_diag);
-	needed = quota_check_needed;
-	gfarm_mutex_unlock(&quota_check_mutex, diag, quota_check_mutex_diag);
+	gfarm_mutex_lock(&quota_check_ctl.mutex,
+	    diag, quota_check_ctl.mutex_diag);
+	needed = quota_check_ctl.needed;
+	gfarm_mutex_unlock(&quota_check_ctl.mutex,
+	    diag, quota_check_ctl.mutex_diag);
 	return (needed);
 }
 
 static void
 quota_check_needed_clear(const char *diag)
 {
-	gfarm_mutex_lock(&quota_check_mutex, diag, quota_check_mutex_diag);
-	quota_check_needed = 0;
-	gfarm_mutex_unlock(&quota_check_mutex, diag, quota_check_mutex_diag);
+	gfarm_mutex_lock(&quota_check_ctl.mutex,
+	    diag, quota_check_ctl.mutex_diag);
+	quota_check_ctl.needed = 0;
+	gfarm_mutex_unlock(&quota_check_ctl.mutex,
+	    diag, quota_check_ctl.mutex_diag);
 }
 
 static int
@@ -812,49 +1305,16 @@ quota_check_main(void)
 	return (0); /* finished */
 }
 
-static void *
-quota_check(void *arg)
-{
-	static const char diag[] = "quota_check";
-
-	(void)gfarm_pthread_set_priority_minimum(diag);
-
-	for (;;) {
-		gfarm_mutex_lock(&quota_check_mutex, diag,
-		    quota_check_mutex_diag);
-		quota_check_running = 0;
-		gfarm_cond_signal(&quota_check_end, diag, quota_check_end_diag);
-
-		while (!quota_check_needed)
-			gfarm_cond_wait(&quota_check_wakeup,
-			    &quota_check_mutex, diag,
-			    quota_check_wakeup_diag);
-
-		quota_check_running = 1;
-		gfarm_mutex_unlock(&quota_check_mutex, diag,
-		    quota_check_mutex_diag);
-
-		gflog_info(GFARM_MSG_1004297, "quota_check: start");
-		quota_check_needed_clear(diag);
-		while (quota_check_main()) {
-			quota_check_needed_clear(diag);
-			gflog_info(GFARM_MSG_1004298, "quota_check: retry");
-		}
-	}
-
-	return (NULL);
-}
-
 static void
-quota_check_wait_for_end(void)
+quota_check_main_loop(struct quota_check_control *ctl)
 {
-	static const char diag[] = "quota_check_wait_for_end";
+	static const char diag[] = "quota_check_main_loop";
 
-	gfarm_mutex_lock(&quota_check_mutex, diag, quota_check_mutex_diag);
-	while (quota_check_running)
-		gfarm_cond_wait(&quota_check_end,
-		    &quota_check_mutex, diag, quota_check_end_diag);
-	gfarm_mutex_unlock(&quota_check_mutex, diag, quota_check_mutex_diag);
+	gflog_info(GFARM_MSG_1004297, "quota_check: start");
+	while (quota_check_main()) {
+		quota_check_needed_clear(diag);
+		gflog_info(GFARM_MSG_1004298, "quota_check: retry");
+	}
 }
 
 static void
@@ -862,41 +1322,278 @@ quota_check_retry_if_running(void)
 {
 	static const char diag[] = "quota_check_retry_if_running";
 
-	gfarm_mutex_lock(&quota_check_mutex, diag, quota_check_mutex_diag);
-	if (quota_check_running)
-		quota_check_needed = 1;
+	gfarm_mutex_lock(&quota_check_ctl.mutex,
+	    diag, quota_check_ctl.mutex_diag);
+	if (quota_check_ctl.running)
+		quota_check_ctl.needed = 1;
 	/* else: cond_wait now */
-	gfarm_mutex_unlock(&quota_check_mutex, diag, quota_check_mutex_diag);
+	gfarm_mutex_unlock(&quota_check_ctl.mutex,
+	    diag, quota_check_ctl.mutex_diag);
+}
+
+
+/*
+ * dirquota_check
+ */
+
+
+/* this is protected by giant_lock */
+static struct dirquota_check_state {
+	struct uint64_to_uint64_map *hardlink_counters;
+
+	int giant_lock_limit;
+	int handled_inodes;
+	int handled_quota_dirs;
+	int handled_dirsets;
+	int retried_dirsets;
+	int skipped_dirsets;
+} dirquota_check_state;
+
+static int
+dirquota_check_needed_per_dirset(void *closure, struct dirset *ds)
+{
+	int *neededp = closure;
+
+	if (*neededp)
+		return (1); /* interrupted */
+	if (!dirquota_is_checked(dirset_get_dirquota(ds))) {
+		*neededp = 1;
+		return (1); /* interrupted */
+	}
+	return (0);
+}
+
+static int
+dirquota_check_needed(void)
+{
+	int needed = 0;
+
+	if (dirquota_invalidate_all_requested)
+		return (1);
+	dirset_foreach_interruptible(
+	    &needed, dirquota_check_needed_per_dirset);
+	return (needed);
+}
+
+static int
+dirquota_invalidate_per_dirset(void *closure, struct dirset *ds)
+{
+	struct dirquota *dq = dirset_get_dirquota(ds);
+
+	dq->invalidate_requested = 1;
+	return (0);
+}
+
+static enum inode_scan_choice
+dirquota_check_per_inode(void *closure, struct inode *inode)
+{
+	struct dirset *ds = closure;
+	struct dirquota *dq = dirset_get_dirquota(ds);
+	gfarm_uint64_t n;
+
+	if (dirquota_invalidate_all_requested || dq->invalidate_requested)
+		return (INODE_SCAN_INTERRUPT);
+
+	if (!inode_is_file(inode) || inode_get_nlink(inode) <= 1) {
+		n = 1;
+	} else {
+		if (!uint64_to_uint64_map_inc_value(
+		    dirquota_check_state.hardlink_counters,
+		    inode_get_number(inode), &n)) {
+			gflog_error(GFARM_MSG_1004637,
+			    "dirquota_check: no memory for %lld hardlinks",
+			    (long long)uint64_to_uint64_map_size(
+			    dirquota_check_state.hardlink_counters));
+			return (INODE_SCAN_INTERRUPT);
+		}
+	}
+
+	if (n == 1) { /* count hard-linked files only at once */
+		dirquota_usage_tmp_update(ds, inode);
+		++dirquota_check_state.handled_inodes;
+	}
+
+	if (++dirquota_check_state.giant_lock_limit
+	    >= QUOTA_CHECK_INODE_STEP) {
+		dirquota_check_state.giant_lock_limit = 0;
+		return (INODE_SCAN_RELEASE_GIANT_LOCK);
+	}
+	return (INODE_SCAN_CONTINUE);
+}
+
+static int
+dirquota_check_per_quota_dir(void *closure, struct quota_dir *qd)
+{
+	struct dirset *ds = closure;
+	struct inode *inode = inode_lookup(quota_dir_get_inum(qd));
+	int interrupted;
+
+	interrupted = inode_foreach_in_subtree_interruptible(
+	    inode, ds, dirquota_check_per_inode, NULL);
+	++dirquota_check_state.handled_quota_dirs;
+	return (interrupted);
+}
+
+static int
+dirquota_check_per_dirset(void *closure, struct dirset *ds)
+{
+	struct dirquota *dq = dirset_get_dirquota(ds);
+	int interrupted;
+
+	if (dirquota_invalidate_all_requested)
+		return (1); /* interrupted */
+
+	if (dq->invalidate_requested) {
+		dq->invalidate_requested = 0;
+		dq->qmm.usage_is_valid = 0;
+	}
+	if (dq->qmm.usage_is_valid) {
+		++dirquota_check_state.skipped_dirsets;
+		return (0);
+	}
+
+	dq->dirquota_checking = 1;
+	quota_usage_clear(&dq->usage_tmp);
+
+	dirquota_check_state.hardlink_counters = uint64_to_uint64_map_new();
+	if (dirquota_check_state.hardlink_counters == NULL) {
+		gflog_error(GFARM_MSG_1004638,
+		    "dirquota_check: no memory for hardlink counter");
+		interrupted = 1;
+	} else {
+		interrupted = dirset_foreach_quota_dir_interruptible(
+		    ds, ds, dirquota_check_per_quota_dir) ||
+		    dq->invalidate_requested ||
+		    dirquota_invalidate_all_requested;
+	}
+	dq->dirquota_checking = 0;
+
+	if (interrupted) {
+		++dirquota_check_state.retried_dirsets;
+	} else {
+		++dirquota_check_state.handled_dirsets;
+		dq->qmm.q.usage = dq->usage_tmp;
+		dq->qmm.usage_is_valid = 1;
+		dirquota_softlimit_exceed(&dq->qmm.q, ds);
+	}
+
+	uint64_to_uint64_map_free(dirquota_check_state.hardlink_counters);
+	dirquota_check_state.hardlink_counters = NULL;
+
+	return (interrupted);
+}
+
+static void
+dirquota_check_run(void)
+{
+	if (dirquota_invalidate_all_requested) {
+		dirset_foreach_interruptible(
+		    NULL, dirquota_invalidate_per_dirset);
+		dirquota_invalidate_all_requested = 0;
+	}
+	dirset_foreach_interruptible(NULL, dirquota_check_per_dirset);
+}
+
+static void
+dirquota_check_main(struct quota_check_control *ctl)
+{
+	time_t time_start, time_total;
+
+	gflog_info(GFARM_MSG_1004639, "dirquota_check: start");
+	time_start = time(NULL);
+
+	giant_lock();
+
+	dirquota_check_state.giant_lock_limit = 0;
+	dirquota_check_state.handled_inodes = 0;
+	dirquota_check_state.handled_quota_dirs = 0;
+	dirquota_check_state.handled_dirsets = 0;
+	dirquota_check_state.retried_dirsets = 0;
+	dirquota_check_state.skipped_dirsets = 0;
+
+	while (dirquota_check_needed())
+		dirquota_check_run();
+
+	giant_unlock();
+
+	time_total = time(NULL) - time_start;
+	gflog_info(GFARM_MSG_1004640,
+	    "dirquota_check: finished, inodes=%lld, quota_dirs=%lld, "
+	    "dirsets=%lld, dirset_retries=%lld, dirsets_ok=%lld time=%lld",
+	    (long long)dirquota_check_state.handled_inodes,
+	    (long long)dirquota_check_state.handled_quota_dirs,
+	    (long long)dirquota_check_state.handled_dirsets,
+	    (long long)dirquota_check_state.retried_dirsets,
+	    (long long)dirquota_check_state.skipped_dirsets,
+	    (long long)time_total);
+}
+
+static struct quota_check_control dirquota_check_ctl = {
+	dirquota_check_main,
+	PTHREAD_MUTEX_INITIALIZER,
+	PTHREAD_COND_INITIALIZER,
+	PTHREAD_COND_INITIALIZER,
+	0,
+	0,
+	0,
+	"dirquota_check_mutex",
+	"dirquota_check_wakeup",
+	"dirquota_check_end",
+};
+
+/* PREREQUISITE: giant_lock */
+void
+dirquota_invalidate(struct dirset *tdirset)
+{
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET) {
+		struct dirquota *dq = dirset_get_dirquota(tdirset);
+
+		dq->invalidate_requested = 1;
+	}
 }
 
 void
-quota_check_start(void)
+dirquota_fixup_schedule(void)
 {
-	static const char diag[] = "quota_check_start";
-
-	gfarm_mutex_lock(&quota_check_mutex, diag, quota_check_mutex_diag);
-	quota_check_needed = 1;
-	quota_check_running = 1;
-	gfarm_cond_signal(&quota_check_wakeup, diag, quota_check_wakeup_diag);
-	gfarm_mutex_unlock(&quota_check_mutex, diag, quota_check_mutex_diag);
+	quota_check_schedule(&dirquota_check_ctl,
+	    time(NULL) +  gfarm_directory_quota_check_start_delay);
 }
 
+/* PREREQUISITE: giant_lock */
+void
+dirquota_check_schedule(void)
+{
+	dirquota_invalidate_all_requested = 1;
+	dirquota_fixup_schedule();
+}
+
+
+/*
+ * thread startup
+ */
+
 void
 quota_check_init(void)
 {
 	gfarm_error_t e;
 
-	if ((e = create_detached_thread(quota_check, NULL))
-	    != GFARM_ERR_NO_ERROR)
+	e = quota_check_at_the_start(&quota_check_ctl);
+	if (e != GFARM_ERR_NO_ERROR)
 		gflog_fatal(GFARM_MSG_1004299,
-		    "create_detached_thread(quota_check_init): %s",
+		    "create_detached_thread(quota_check): %s",
 		    gfarm_error_string(e));
 
-	quota_check_start();
-	quota_check_wait_for_end();
+	e = quota_check_at_the_start(&dirquota_check_ctl);
+	if (e != GFARM_ERR_NO_ERROR)
+		gflog_fatal(GFARM_MSG_1004641,
+		    "create_detached_thread(dirquota_check): %s",
+		    gfarm_error_string(e));
 }
 
-/* server operations */
+/*
+ * server operations
+ */
+
 static gfarm_error_t
 quota_get_common(struct peer *peer, int from_client, int skip, int is_group)
 {
@@ -1183,8 +1880,11 @@ gfm_server_quota_check(struct peer *peer, int from_client, int skip)
 	}
 	giant_unlock();
 
-	quota_check_start();
-	quota_check_wait_for_end();
+	quota_check_start(&quota_check_ctl);
+	quota_check_wait_for_end(&quota_check_ctl);
+
+	quota_check_start(&dirquota_check_ctl);
+	quota_check_wait_for_end(&dirquota_check_ctl);
 
 	return (gfm_server_put_reply(peer, diag, e, ""));
 }
