@@ -1,5 +1,5 @@
 /*
- * $Id: replica_check.c 9969 2015-12-08 08:25:51Z tatebe $
+ * $Id: replica_check.c 10774 2018-02-13 13:07:50Z n-soda $
  */
 
 #include <stdio.h>
@@ -10,6 +10,7 @@
 #include <errno.h>
 #include <pthread.h>
 #include <sys/time.h>
+#include <time.h>
 #include <string.h>
 #include <syslog.h>
 #include <sched.h>
@@ -28,6 +29,7 @@
 #include "auth.h"
 #include "repattr.h"
 #include "gfm_proto.h"
+#include "timespec.h"
 
 #include "db_access.h"
 #include "rpcsubr.h"
@@ -35,6 +37,7 @@
 #include "fsngroup.h"
 #include "gfmd.h"
 #include "inode.h"
+#include "dirset.h"
 #include "process.h"
 #include "dead_file_copy.h"
 #include "dir.h"
@@ -77,11 +80,62 @@ struct replication_info {
 	gfarm_ino_t inum;
 	gfarm_uint64_t gen;
 	struct replica_spec replica_spec;
+	struct dirset *tdirset;
 };
 
+static int replica_check_ctrl_enabled(void);
 static int replica_check_remove_enabled(void);
 static int replica_check_reduced_log_enabled(void);
 
+static int
+replica_check_remove_grace_used_space_ratio_locked(void)
+{
+	int rv;
+
+	config_var_lock();
+	rv = gfarm_replica_check_remove_grace_used_space_ratio;
+	config_var_unlock();
+	return (rv);
+}
+
+static int
+replica_check_remove_grace_time_locked(void)
+{
+	int rv;
+
+	config_var_lock();
+	rv = gfarm_replica_check_remove_grace_time;
+	config_var_unlock();
+	return (rv);
+}
+
+static int
+replica_check_host_down_thresh_locked(void)
+{
+	int rv;
+
+	config_var_lock();
+	rv = gfarm_replica_check_host_down_thresh;
+	config_var_unlock();
+	if (rv < 0)
+		rv = 0;
+	return (rv);
+}
+
+static int
+replica_check_minimum_interval_locked(void)
+{
+	int rv;
+
+	config_var_lock();
+	rv = gfarm_replica_check_minimum_interval;
+	config_var_unlock();
+	if (rv < 0)
+		rv = 0;
+	return (rv);
+}
+
+
 #define REDUCED_WARN(msg_no, state, ...)				\
 	{								\
 		if (replica_check_reduced_log_enabled())		\
@@ -98,17 +152,94 @@ static int replica_check_reduced_log_enabled(void);
 			gflog_info(msg_no, __VA_ARGS__);		\
 	}
 
+static int sleep_time;  /* from gfarm_replica_check_sleep_time */
+static double lock_sleep_time;  /* total */
+static double retry_sleep_time; /* total */
+
+static int yield_time;  /* from gfarm_replica_check_yield_time */
+static double total_yield_time;
+
+static gfarm_uint64_t req_ok_num_total, req_ok_size_total;
+static gfarm_uint64_t remove_num_total, remove_size_total;
+
+/*
+ * 0: The replica should not be removed.
+ * 1: The replica may be removed. (try inode_remove_replica_protected)
+ */
+static int
+replica_check_remove_grace_is_over(struct inode *inode,
+	struct host *replica_host)
+{
+	float used_space = host_status_get_disk_usage_percent(replica_host);
+	struct gfarm_timespec *atime = inode_get_atime(inode);
+	struct gfarm_timespec sub;
+	struct gfarm_timespec grace_time;
+	struct timespec now_timespec;
+
+#ifdef DEBUG_REPLICA_CHECK
+	RC_LOG_DEBUG(GFARM_MSG_1005013,
+	    "replica_check_remove_grace_is_over: "
+	    "inum=%lld, replica=%s, used_space=%f, grace_used_space_ratio=%d",
+	    (long long)inode_get_number(inode),
+	    host_name(replica_host), used_space,
+	    gfarm_replica_check_remove_grace_used_space_ratio);
+#endif
+	/*
+	 * gfarm_replica_check_remove_grace_used_space_ratio is
+	 * giant_lock()ed.
+	 */
+	if (used_space <=
+	    (float)gfarm_replica_check_remove_grace_used_space_ratio)
+		return (0);
+
+	/*
+	 * gfarm_replica_check_remove_grace_time is giant_lock()ed.
+	 */
+	grace_time.tv_sec = gfarm_replica_check_remove_grace_time;
+	grace_time.tv_nsec = 0;
+
+	gfarm_gettime(&now_timespec);
+	sub.tv_sec = now_timespec.tv_sec;
+	sub.tv_nsec = now_timespec.tv_nsec;
+
+	gfarm_timespec_sub(&sub, atime);
+	if (gfarm_timespec_cmp(&sub, &grace_time) > 0) {
+		/* (now - atime) > grace_time */
+#ifdef DEBUG_REPLICA_CHECK
+		RC_LOG_DEBUG(GFARM_MSG_1005014,
+		    "replica_check_remove_grace_is_over: "
+		    "inum=%lld, replica=%s: may be removed",
+		    (long long)inode_get_number(inode),
+		    host_name(replica_host));
+#endif
+		return (1); /* remove a surplus replica immediately */
+	}
+
+	return (0);
+}
+
+static int
+replica_check_remove_grace_disabled(struct inode *inode,
+	struct host *replica_host)
+{
+	return (1); /* remove all surplus replicas immediately */
+}
+
 static gfarm_error_t
 replica_check_remove_replicas(struct inode *inode,
 	int n_srcs, struct host **srcs, struct replication_info *info)
 {
 	gfarm_error_t e, e2;
 	int i;
+	int (*grace_is_over_func)(struct inode *, struct host *);
 
 	if (n_srcs <= 1)
 		return (GFARM_ERR_NO_ERROR); /* skip */
 	if (n_srcs <= info->replica_spec.desired_number)
 		return (GFARM_ERR_NO_ERROR); /* skip */
+	if (gfarm_replica_check_remove_grace_used_space_ratio >= 100)
+		return (GFARM_ERR_NO_ERROR); /* skip */
+
 	if (inode_get_nlink(inode) >= 2) {
 		char *repattr;
 		int desired_number;
@@ -129,6 +260,17 @@ replica_check_remove_replicas(struct inode *inode,
 	/* prefer a host which has few disk_avail */
 	host_sort_to_remove_replicas(n_srcs, srcs);
 
+	/*
+	 * gfarm_replica_check_remove_grace_used_space_ratio and
+	 * gfarm_replica_check_remove_grace_time are giant_lock()ed.
+	 */
+	/* shortcut */
+	if (gfarm_replica_check_remove_grace_used_space_ratio <= 0 &&
+	    gfarm_replica_check_remove_grace_time <= 0)
+		grace_is_over_func = replica_check_remove_grace_disabled;
+	else
+		grace_is_over_func = replica_check_remove_grace_is_over;
+
 	e = GFARM_ERR_NO_ERROR;
 	/* srcs are available replicas */
 	for (i = 0; i < n_srcs; i++) {
@@ -143,15 +285,20 @@ replica_check_remove_replicas(struct inode *inode,
 		    i, host_name(srcs[i]), (long long)used, (long long)avail);
 #endif
 #endif
+		if (grace_is_over_func(inode, srcs[i]) == 0)
+			continue; /* skip */
+
 		/* remove a replica only when it is excessive */
 		e2 = inode_remove_replica_protected(inode, srcs[i],
-		    &info->replica_spec);
+		    &info->replica_spec, info->tdirset);
 		if (e2 == GFARM_ERR_NO_ERROR) {
 			REDUCED_INFO(GFARM_MSG_1004273, &remove_ok_state,
 			    "replica_check: %lld:%lld:%s@%s: removed",
 			    (long long)info->inum, (long long)info->gen,
 			    user_name(inode_get_user(inode)),
 			    host_name(srcs[i]));
+			remove_num_total++;
+			remove_size_total += inode_get_size(inode);
 		}
 
 		/* ignore error except GFARM_ERR_FILE_BUSY */
@@ -203,6 +350,7 @@ replica_check_fix(struct replication_info *info)
 	struct inode *inode = inode_lookup(info->inum);
 	int n_srcs, n_existing, n_being_removed, transaction = 0;
 	struct host **srcs, **existing, **being_removed;
+	int req_ok_num = 0;
 	static const char diag[] = "replica_check_fix";
 
 	if (inode == NULL || !inode_is_file(inode) ||
@@ -283,16 +431,19 @@ replica_check_fix(struct replication_info *info)
 
 	/* create replicas if the replicas are insufficient */
 	e = inode_schedule_replication(
-	    inode, 1,
+	    inode, info->tdirset, 1,
 	    info->replica_spec.desired_number, info->replica_spec.repattr,
 	    n_srcs, srcs, &n_existing, existing,
-	    gfarm_replica_check_host_down_thresh,
-	    &n_being_removed, being_removed, diag);
+	    gfarm_replica_check_host_down_thresh, /* giant_lock()ed */
+	    &n_being_removed, being_removed, diag, &req_ok_num);
 
 	if (e == GFARM_ERR_NO_ERROR && replica_check_remove_enabled())
 		/* remove excessive number of replicas */
 		e = replica_check_remove_replicas(inode, n_srcs, srcs, info);
 
+	req_ok_num_total += req_ok_num;
+	req_ok_size_total += inode_get_size(inode) * req_ok_num;
+
 	if (transaction)
 		db_end(diag);
 	free(existing);
@@ -338,16 +489,21 @@ replica_check_stack_init(void)
 }
 
 static void
-replica_check_stack_push(struct inode *dir_ino, struct inode *file_ino)
+replica_check_stack_push(struct inode *dir_ino, struct inode *file_ino,
+	struct dirset *tdirset)
 {
+	struct replication_info *rep_info =
+	    &replica_check_stack[replica_check_stack_index];
+
 	assert(replica_check_stack_index < replica_check_stack_size);
 
-	replica_check_stack[replica_check_stack_index].inum
-		= inode_get_number(file_ino);
-	replica_check_stack[replica_check_stack_index].gen
-		= inode_get_gen(file_ino);
-	replica_check_desired_set(dir_ino, file_ino,
-	    &(replica_check_stack[replica_check_stack_index]));
+	rep_info->inum = inode_get_number(file_ino);
+	rep_info->gen = inode_get_gen(file_ino);
+	replica_check_desired_set(dir_ino, file_ino, rep_info);
+	rep_info->tdirset = tdirset;
+	if (tdirset != TDIRSET_IS_UNKNOWN && tdirset != TDIRSET_IS_NOT_SET)
+		dirset_inc_busy_count(tdirset);
+
 	replica_check_stack_index++;
 }
 
@@ -362,20 +518,47 @@ replica_check_stack_pop(struct replication_info *infop)
 }
 
 static void
-replica_check_giant_lock_default(void)
+replication_info_free(struct replication_info *rep_info)
 {
+	if (rep_info->tdirset != TDIRSET_IS_UNKNOWN &&
+	    rep_info->tdirset != TDIRSET_IS_NOT_SET)
+		dirset_dec_busy_count(rep_info->tdirset);
+	replica_spec_free(&rep_info->replica_spec);
+}
+
+static void
+sleep_time_update(int *varp, int nanosec)
+{
+	if (nanosec > GFARM_SECOND_BY_NANOSEC)
+		nanosec = GFARM_SECOND_BY_NANOSEC;
+	else if (nanosec < 0)
+		nanosec = 0;
+
+	*varp = nanosec;
+}
+
+static void
+replica_check_giant_lock(void)
+{
+	if (yield_time > 0) {
+		gfarm_nanosleep(yield_time);
+		total_yield_time += .000000001 * yield_time;
+	}
 	if (!giant_trylock()) {
 #ifdef DEBUG_REPLICA_CHECK
 		RC_LOG_DEBUG(GFARM_MSG_1004276,
 		    "replica_check: cannot get lock, sleep");
 #endif
-		gfarm_nanosleep(gfarm_replica_check_sleep_time);
+		gfarm_nanosleep(sleep_time);
+		lock_sleep_time += .000000001 * sleep_time;
+
 		giant_lock();
+		/* gfarm_replica_check_sleep_time is giant_lock()ed */
+		sleep_time_update(&sleep_time, gfarm_replica_check_sleep_time);
+		sleep_time_update(&yield_time, gfarm_replica_check_yield_time);
 	}
 }
 
-static void (*replica_check_giant_lock)(void);
-
 #define YIELD_INTERVAL 0  /* 0: every time */
 static int yield_count = 0;
 
@@ -391,8 +574,36 @@ replica_check_giant_unlock_yield(void)
 
 static void (*replica_check_giant_unlock)(void) = giant_unlock;
 
+static void
+replica_check_giant_lock_init()
+{
+	gfarm_error_t e;
+	int stime, ytime;
+
+	e = gfarm_pthread_set_priority_minimum("replica_check");
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_info(GFARM_MSG_1004279,
+		    "replica_check: use sched_yield()");
+		replica_check_giant_unlock = replica_check_giant_unlock_yield;
+	}
+
+	config_var_lock();
+	stime = gfarm_replica_check_sleep_time;
+	ytime = gfarm_replica_check_yield_time;
+	config_var_unlock();
+	sleep_time_update(&sleep_time, stime);
+	sleep_time_update(&yield_time, ytime);
+}
+
+/*
+ * these variables are protected by giant lock since they are also
+ * accessed by a signal handler
+ */
+static gfarm_ino_t info_inum, info_table_size;
+static time_t info_time_start;
+
 static int
-replica_check_main_dir(gfarm_ino_t inum, gfarm_ino_t *countp)
+replica_check_main_dir(gfarm_ino_t inum, gfarm_ino_t *countp, int *stopped)
 {
 	gfarm_error_t e;
 	struct inode *dir_ino, *file_ino;
@@ -400,11 +611,14 @@ replica_check_main_dir(gfarm_ino_t inum, gfarm_ino_t *countp)
 	DirCursor cursor;
 	gfarm_off_t dir_offset = 0;
 	DirEntry entry;
+	struct dirset *tdirset;
 	struct replication_info rep_info;
 	int need_to_retry = 0, eod = 0, i;
 
+	*stopped = 0;
 	while (!eod) {
 		replica_check_giant_lock();
+		info_inum = inum;
 		dir_ino = inode_lookup(inum);
 		if (dir_ino == NULL) {
 			replica_check_giant_unlock();
@@ -415,10 +629,16 @@ replica_check_main_dir(gfarm_ino_t inum, gfarm_ino_t *countp)
 			replica_check_giant_unlock();
 			return (need_to_retry);
 		}
+		if (!replica_check_ctrl_enabled()) { /* gfrepcheck stop */
+			*stopped = 1;
+			replica_check_giant_unlock();
+			return (need_to_retry);
+		}
 		if (!dir_cursor_set_pos(dir, dir_offset, &cursor)) {
 			replica_check_giant_unlock();
 			return (need_to_retry);
 		}
+		tdirset = inode_search_tdirset(dir_ino);
 		/* avoid long giant lock */
 		for (i = 0; i < REPLICA_CHECK_DIRENTS_BUFCOUNT; i++) {
 			entry = dir_cursor_get_entry(dir, &cursor);
@@ -428,7 +648,8 @@ replica_check_main_dir(gfarm_ino_t inum, gfarm_ino_t *countp)
 			}
 			file_ino = dir_entry_get_inode(entry);
 			if (inode_is_file(file_ino))
-				replica_check_stack_push(dir_ino, file_ino);
+				replica_check_stack_push(dir_ino, file_ino,
+				    tdirset);
 			if (!dir_cursor_next(dir, &cursor)) {
 				eod = 1; /* end of directory */
 				break;
@@ -450,6 +671,7 @@ replica_check_main_dir(gfarm_ino_t inum, gfarm_ino_t *countp)
 					break; /* success or error */
 				/* retry */
 				gfarm_nanosleep(sl);
+				retry_sleep_time += .000000001 * sl;
 				if (sl < GFARM_SECOND_BY_NANOSEC)
 					sl *= 2; /* 2,4,8,...,512,1024,1024 */
 			}
@@ -461,21 +683,22 @@ replica_check_main_dir(gfarm_ino_t inum, gfarm_ino_t *countp)
 				    gfarm_error_string(e));
 			}
 			(*countp)++;
-			replica_spec_free(&rep_info.replica_spec);
+			replication_info_free(&rep_info);
 		}
 	}
 	return (need_to_retry);
 }
 
-static gfarm_ino_t info_inum, info_table_size;
-static time_t info_time_start;
+#define REPLICA_CHECK_INTERRUPT_STEP 10000
+
+#define ENABLE_STR(b) ((b) ? "enable" : "disable")
 
 static int
 replica_check_main(void)
 {
 	gfarm_ino_t inum, table_size, count = 0;
 	gfarm_ino_t root_inum = inode_root_number();
-	int need_to_retry = 0;
+	int need_to_retry = 0, stopped;
 	time_t time_total, time_start;
 
 	replica_check_giant_lock();
@@ -483,19 +706,36 @@ replica_check_main(void)
 	info_time_start = time_start = time(NULL);
 	replica_check_giant_unlock();
 
+	lock_sleep_time = retry_sleep_time = 0;
+	total_yield_time = 0;
+	req_ok_num_total = req_ok_size_total = 0;
+	remove_num_total = remove_size_total = 0;
 	RC_LOG_INFO(GFARM_MSG_1003632, "replica_check: start");
 	RC_LOG_INFO(GFARM_MSG_1004277,
 	    "replica_check: remove=%s, reduced_log=%s",
-	    replica_check_remove_enabled() ? "enable" : "disable",
-	    replica_check_reduced_log_enabled() ? "enable" : "disable");
+	    ENABLE_STR(replica_check_remove_enabled()),
+	    ENABLE_STR(replica_check_reduced_log_enabled()));
+	RC_LOG_INFO(GFARM_MSG_1005015,
+	    "replica_check: remove_grace_used_space_ratio=%d, "
+	    "remove_grace_time=%d",
+	    replica_check_remove_grace_used_space_ratio_locked(),
+	    replica_check_remove_grace_time_locked());
 
 	for (inum = root_inum;;) {
-		replica_check_giant_lock();
-		info_inum = inum;
-		replica_check_giant_unlock();
+		if (inum % REPLICA_CHECK_INTERRUPT_STEP == 0 &&
+		    !replica_check_ctrl_enabled()) { /* gfrepcheck stop */
+			RC_LOG_INFO(GFARM_MSG_1005016,
+			    "replica_check: stopped (interrupted)");
+			break;
+		}
 
-		if (replica_check_main_dir(inum, &count))
+		if (replica_check_main_dir(inum, &count, &stopped))
 			need_to_retry = 1;
+		if (stopped) { /* gfrepcheck stop */
+			RC_LOG_INFO(GFARM_MSG_1005017,
+			    "replica_check: stopped (interrupted)");
+			break;
+		}
 		inum++; /* a next directory */
 		if (inum >= table_size) {
 			replica_check_giant_lock();
@@ -507,17 +747,33 @@ replica_check_main(void)
 		}
 	}
 	time_total = time(NULL) - time_start;
-	RC_LOG_INFO(GFARM_MSG_1003633,
-	    "replica_check: finished, files=%llu, time=%lld",
-	    (unsigned long long)count, (long long)time_total);
+	RC_LOG_INFO(GFARM_MSG_1005040,
+	    "replica_check: finished, table=%llu, inum=%llu, file=%llu, "
+	    "rep num=%llu, rep size=%llu, remove num=%llu, remove size=%llu, "
+	    "time=%lld (%.3fh), lock sleep=%g, retry sleep=%g",
+	    (unsigned long long)table_size,
+	    (unsigned long long)inum,
+	    (unsigned long long)count,
+	    (unsigned long long)req_ok_num_total,
+	    (unsigned long long)req_ok_size_total,
+	    (unsigned long long)remove_num_total,
+	    (unsigned long long)remove_size_total,
+	    (long long)time_total, (float)time_total / 60 / 60,
+	    lock_sleep_time, retry_sleep_time);
+	if (total_yield_time != 0) { /* hide undocumented unless it's set */
+		RC_LOG_INFO(GFARM_MSG_1005049,
+		    "replica_check: yield sleep=%g",
+		    total_yield_time);
+	}
 
 	replica_check_giant_lock();
-	info_time_start = 0;
+	info_time_start = 0;  /* stopped */
 	replica_check_giant_unlock();
 
 	return (need_to_retry);
 }
 
+/* this function is called from sigs_handler() in gfmd.c */
 void
 replica_check_info(void)
 {
@@ -583,6 +839,7 @@ replica_check_targets_init(void)
 	return (1);
 }
 
+/* do not call giant_lock() in this */
 static void
 replica_check_targets_add(time_t sec)
 {
@@ -605,9 +862,11 @@ replica_check_targets_add(time_t sec)
 #endif
 }
 
-/* not delete */
+/* do not call giant_lock() in this */
+/* next target is not deleted */
 static int
-replica_check_targets_next(struct timeval *next, const struct timeval *now)
+replica_check_targets_next(struct timeval *next, const struct timeval *now,
+	int minimum_interval)
 {
 	size_t i;
 	struct timeval now2;
@@ -636,7 +895,7 @@ replica_check_targets_next(struct timeval *next, const struct timeval *now)
 	now2 = *now;
 
 	/* integrate near target times */
-	now2.tv_sec += gfarm_replica_check_minimum_interval;
+	now2.tv_sec += minimum_interval;
 
 	/* assert(targets_num >= 2); */
 	for (i = targets_num - 1;; i--) {
@@ -689,11 +948,14 @@ replica_check_cond_wait(void)
 {
 	static const char diag[] = "replica_check_cond_wait";
 	struct timeval next, now;
+	int minimum_interval = replica_check_minimum_interval_locked();
 
 	gfarm_mutex_lock(&replica_check_mutex, diag, REPLICA_CHECK_DIAG);
+	/* do not call giant_lock() in replica_check_mutex */
 	for (;;) {
 		gettimeofday(&now, NULL);
-		if (replica_check_targets_next(&next, &now)) {
+		if (replica_check_targets_next(
+		    &next, &now, minimum_interval)) {
 #ifdef DEBUG_REPLICA_CHECK
 			{
 				struct timeval after;
@@ -727,8 +989,7 @@ replica_check_cond_wait(void)
 		 */
 		gfarm_mutex_unlock(
 		    &replica_check_mutex, diag, REPLICA_CHECK_DIAG);
-		gfarm_nanosleep(
-		    (unsigned long long)gfarm_replica_check_minimum_interval *
+		gfarm_nanosleep((unsigned long long)minimum_interval *
 		    GFARM_SECOND_BY_NANOSEC);
 		gfarm_mutex_lock(
 		    &replica_check_mutex, diag, REPLICA_CHECK_DIAG);
@@ -742,6 +1003,7 @@ static void
 replica_check_cond_signal(const char *diag, long sec)
 {
 	gfarm_mutex_lock(&replica_check_mutex, diag, REPLICA_CHECK_DIAG);
+	/* do not call giant_lock() in replica_check_mutex */
 	if (replica_check_initialized) {
 #ifdef DEBUG_REPLICA_CHECK
 		RC_LOG_DEBUG(GFARM_MSG_1003639,
@@ -772,8 +1034,8 @@ replica_check_start_host_down(void)
 {
 	static const char diag[] = "replica_check_start_host_down";
 
-	replica_check_cond_signal(
-	    diag, gfarm_replica_check_host_down_thresh);
+	replica_check_cond_signal(diag,
+	    replica_check_host_down_thresh_locked());
 	/* NOTE: execute replica_check_main() twice after restarting gfsd */
 }
 
@@ -817,6 +1079,14 @@ replica_check_start_fsngroup_modify(void)
 	replica_check_cond_signal(diag, 60);  /* 60 sec. */
 }
 
+void
+replica_check_start_host_is_not_busy(void)
+{
+	static const char diag[] = "replica_check_start_host_is_not_busy";
+
+	replica_check_cond_signal(diag, 0);
+}
+
 #define ENABLE  1
 #define DISABLE 0
 
@@ -828,6 +1098,7 @@ static int replica_check_ctrl = DISABLE;
 static int replica_check_remove = DISABLE;
 static int replica_check_reduced_log = DISABLE;
 
+/* MAINCTRL */
 static void
 replica_check_ctrl_set(int ctrl)
 {
@@ -847,6 +1118,23 @@ replica_check_ctrl_set(int ctrl)
 	    REPLICA_CHECK_CTRL_DIAG);
 }
 
+/* MAINCTRL */
+static int
+replica_check_ctrl_enabled(void)
+{
+	static const char diag[] = "replica_check_ctrl_enabled";
+	int ctrl;
+
+	gfarm_mutex_lock(&replica_check_ctrl_mutex, diag,
+			 REPLICA_CHECK_CTRL_DIAG);
+	ctrl = replica_check_ctrl;
+	gfarm_mutex_unlock(&replica_check_ctrl_mutex, diag,
+			   REPLICA_CHECK_CTRL_DIAG);
+
+	return (ctrl == ENABLE);
+}
+
+/* MAINCTRL */
 static void
 replica_check_ctrl_wait(void)
 {
@@ -964,10 +1252,10 @@ gfm_server_replica_check_ctrl(struct peer *peer, int from_client, int skip)
 		    "replica_check: ctrl=%s",
 		    replica_check_ctrl_string(ctrl));
 		switch (ctrl) {
-		case GFM_PROTO_REPLICA_CHECK_CTRL_START:
+		case GFM_PROTO_REPLICA_CHECK_CTRL_START: /* MAINCTRL */
 			replica_check_ctrl_set(ENABLE);
 			break;
-		case GFM_PROTO_REPLICA_CHECK_CTRL_STOP:
+		case GFM_PROTO_REPLICA_CHECK_CTRL_STOP: /* MAINCTRL */
 			replica_check_ctrl_set(DISABLE);
 			break;
 		case GFM_PROTO_REPLICA_CHECK_CTRL_REMOVE_ENABLE:
@@ -992,6 +1280,94 @@ gfm_server_replica_check_ctrl(struct peer *peer, int from_client, int skip)
 	return (gfm_server_put_reply(peer, diag, e, ""));
 }
 
+static const char *
+replica_check_status_target_string(gfarm_int32_t status_target)
+{
+	switch (status_target) {
+	case GFM_PROTO_REPLICA_CHECK_STATUS_MAINCTRL:
+		return ("MAINCTRL");
+	case GFM_PROTO_REPLICA_CHECK_STATUS_REMOVE:
+		return ("REMOVE");
+	case GFM_PROTO_REPLICA_CHECK_STATUS_REDUCED_LOG:
+		return ("REDUCED_LOG");
+	default:
+		return ("<UNKNOWN>");
+	}
+}
+
+static gfarm_int32_t
+replica_check_status_mainctrl(void) {
+	gfarm_int32_t rv;
+
+	if (replica_check_ctrl_enabled()) {
+		/* info_time_start needs giant_lock() */
+		if (info_time_start != 0)
+			rv = GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE_RUNNING;
+		else
+			rv = GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE_STOPPED;
+	} else {
+		/* info_time_start needs giant_lock() */
+		if (info_time_start != 0)
+			rv = GFM_PROTO_REPLICA_CHECK_STATUS_DISABLE_RUNNING;
+		else
+			rv = GFM_PROTO_REPLICA_CHECK_STATUS_DISABLE_STOPPED;
+	}
+	return (rv);
+}
+
+gfarm_error_t
+gfm_server_replica_check_status(struct peer *peer, int from_client, int skip)
+{
+	gfarm_error_t e;
+	gfarm_int32_t status_target;
+	gfarm_int32_t status = -1;
+	struct user *user;
+	static const char diag[] = "GFM_PROTO_REPLICA_CHECK_STATUS";
+
+	e = gfm_server_get_request(peer, diag, "i", &status_target);
+	if (e != GFARM_ERR_NO_ERROR)
+		return (e);
+	if (skip)
+		return (GFARM_ERR_NO_ERROR);
+
+	giant_lock();
+	if (!from_client || (user = peer_get_user(peer)) == NULL ||
+	    !user_is_admin(user)) {
+		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
+		gflog_debug(GFARM_MSG_1005041, "%s", gfarm_error_string(e));
+	} else {
+		gflog_debug(GFARM_MSG_1005042,
+		    "replica_check_status: target=%s",
+		    replica_check_status_target_string(status_target));
+		switch (status_target) {
+		case GFM_PROTO_REPLICA_CHECK_STATUS_MAINCTRL:
+			status = replica_check_status_mainctrl();
+			break;
+		case GFM_PROTO_REPLICA_CHECK_STATUS_REMOVE:
+			status = replica_check_remove_enabled() ?
+			    GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE :
+			    GFM_PROTO_REPLICA_CHECK_STATUS_DISABLE;
+			break;
+		case GFM_PROTO_REPLICA_CHECK_STATUS_REDUCED_LOG:
+			status = replica_check_reduced_log_enabled() ?
+			    GFM_PROTO_REPLICA_CHECK_STATUS_ENABLE :
+			    GFM_PROTO_REPLICA_CHECK_STATUS_DISABLE;
+			break;
+		default:
+			e = GFARM_ERR_INVALID_ARGUMENT;
+			gflog_debug(GFARM_MSG_1005043,
+			    "replica_check_status: %s: target=%d",
+			    gfarm_error_string(e), status_target);
+		}
+	}
+	giant_unlock();
+	if (e != GFARM_ERR_NO_ERROR)
+		e = gfm_server_put_reply(peer, diag, e, "");
+	else
+		e = gfm_server_put_reply(peer, diag, e, "i", status);
+	return (e);
+}
+
 /* workaround for #406 - obsolete replicas remain existing */
 #define DFC_SCAN_INTERVAL 21600 /* 6 hours */
 
@@ -999,25 +1375,12 @@ static void *
 replica_check_thread(void *arg)
 {
 	time_t dfc_scan_time = 0;
-	gfarm_error_t e;
 
-	e = gfarm_pthread_set_priority_minimum(REPLICA_CHECK_DIAG);
-	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_info(GFARM_MSG_1004279,
-		    "replica_check: use sched_yield()");
-		replica_check_giant_unlock = replica_check_giant_unlock_yield;
-	}
-
-	if (gfarm_replica_check_sleep_time > 0)
-		replica_check_giant_lock = replica_check_giant_lock_default;
-	else
-		replica_check_giant_lock = giant_lock;
-
-	if (gfarm_replica_check_sleep_time > GFARM_SECOND_BY_NANOSEC)
-		gfarm_replica_check_sleep_time = GFARM_SECOND_BY_NANOSEC;
+	replica_check_giant_lock_init();
 
 	for (;;) {
-		time_t t = time(NULL) + gfarm_replica_check_minimum_interval;
+		time_t t = time(NULL) +
+		    replica_check_minimum_interval_locked();
 
 		replica_check_cond_wait();
 		replica_check_ctrl_wait(); /* if the ctrl is stop, wait here */
