@@ -1,10 +1,15 @@
 extern int debug_mode;
 
 void giant_init(void);
+void giant_type_log(void);
 void giant_lock(void);
 int giant_trylock(void);
 void giant_unlock(void);
 
+void config_var_init(void);
+void config_var_lock(void);
+void config_var_unlock(void);
+
 gfarm_error_t create_detached_thread(void *(*)(void *), void *);
 gfarm_error_t gfarm_pthread_set_priority_minimum(const char *);
 
